@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2010-2020 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2021 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -13,6 +13,7 @@ import string
 
 from xpra.log import Logger
 from xpra.scripts.config import InitExit
+from xpra.common import SPLASH_EXIT_DELAY
 from xpra.child_reaper import getChildReaper, reaper_cleanup
 from xpra.net import compression
 from xpra.net.common import may_log_packet, PACKET_TYPES
@@ -29,12 +30,13 @@ from xpra.version_util import get_version_info, XPRA_VERSION
 from xpra.platform.info import get_name
 from xpra.os_util import (
     get_machine_id, get_user_uuid, register_SIGUSR_signals,
-    load_binary_file, force_quit,
+    filedata_nocrlf, force_quit,
     SIGNAMES, BITS,
     strtobytes, bytestostr, hexstr, monotonic_time, use_tty,
+    parse_encoded_bin_data,
     )
 from xpra.util import (
-    flatten_dict, typedict, updict, parse_simple_dict,
+    flatten_dict, typedict, updict, parse_simple_dict, noerr,
     repr_ellipsized, ellipsizer, nonl,
     envbool, envint, disconnect_is_an_error, dump_all_frames, engs, csv, obsc,
     )
@@ -56,8 +58,9 @@ EXTRA_TIMEOUT = 10
 ALLOW_UNENCRYPTED_PASSWORDS = envbool("XPRA_ALLOW_UNENCRYPTED_PASSWORDS", False)
 ALLOW_LOCALHOST_PASSWORDS = envbool("XPRA_ALLOW_LOCALHOST_PASSWORDS", True)
 DETECT_LEAKS = envbool("XPRA_DETECT_LEAKS", False)
-LEGACY_SALT_DIGEST = envbool("XPRA_LEGACY_SALT_DIGEST", True)
+LEGACY_SALT_DIGEST = envbool("XPRA_LEGACY_SALT_DIGEST", False)
 MOUSE_DELAY = envint("XPRA_MOUSE_DELAY", 0)
+SPLASH_LOG = envbool("XPRA_SPLASH_LOG", False)
 
 
 def noop():
@@ -93,11 +96,13 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
         getChildReaper()
         log("XpraClientBase.defaults_init() os.environ:")
         for k,v in os.environ.items():
-            log(" %s=%s", k, nonl(v))
+            log(" %s=%r", k, v)
         #client state:
         self.exit_code = None
         self.exit_on_signal = False
         self.display_desc = {}
+        self.progress_process = None
+        self.progress_timer = None
         #connection attributes:
         self.hello_extra = {}
         self.compression_level = 0
@@ -146,8 +151,6 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
         self.password = opts.password
         self.password_file = opts.password_file
         self.encryption = opts.encryption or opts.tcp_encryption
-        if self.encryption:
-            crypto_backend_init()
         self.encryption_keyfile = opts.encryption_keyfile or opts.tcp_encryption_keyfile
         self.init_challenge_handlers(opts.challenge_handlers)
         self.init_aliases()
@@ -155,16 +158,48 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
             self.install_signal_handlers()
 
 
+    def show_progress(self, pct, text=""):
+        log("progress(%s, %s)", pct, text)
+        if SPLASH_LOG:
+            log.info("%3i %s", pct, text)
+        pp = self.progress_process
+        if not pp:
+            return
+        if pp.poll():
+            self.progress_process = None
+            return
+        noerr(pp.stdin.write, ("%i:%s\n" % (pct, text)).encode("latin1"))
+        noerr(pp.stdin.flush)
+        if pct==100:
+            #it should exit on its own, but just in case:
+            #kill it if it's still running after 2 seconds
+            self.cancel_progress_timer()
+            def stop_progress():
+                self.progress_timer = None
+                self.stop_progress_process()
+            self.progress_timer = self.timeout_add(SPLASH_EXIT_DELAY*1000+500, stop_progress)
+
+    def cancel_progress_timer(self):
+        pt = self.progress_timer
+        if pt:
+            self.progress_timer = None
+            self.source_remove(pt)
+
+
     def init_challenge_handlers(self, challenge_handlers):
         #register the authentication challenge handlers:
         authlog("init_challenge_handlers(%s)", challenge_handlers)
         ch = tuple(x.strip() for x in (challenge_handlers or "".split(",")))
-        ALL_AUTH_HANDLERS = ("uri", "file", "env", "kerberos", "gss", "u2f", "prompt")
         for ch_name in ch:
             if ch_name=="none":
                 continue
             if ch_name=="all":
-                items = ALL_AUTH_HANDLERS
+                items = (
+                    "uri", "file", "env",
+                    "kerberos", "gss",
+                    "u2f",
+                    "prompt", "prompt", "prompt", "prompt",
+                    )
                 ierror = authlog
             else:
                 items = (ch_name, )
@@ -305,10 +340,11 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
         protocol.receive_aliases.update(self._aliases)
         protocol.enable_default_encoder()
         protocol.enable_default_compressor()
-        if self.encryption and ENCRYPT_FIRST_PACKET:
+        encryption = self.get_encryption()
+        if encryption and ENCRYPT_FIRST_PACKET:
             key = self.get_encryption_key()
-            protocol.set_cipher_out(self.encryption,
-                                          DEFAULT_IV, key, DEFAULT_SALT, DEFAULT_ITERATIONS, INITIAL_PADDING)
+            protocol.set_cipher_out(encryption,
+                                    DEFAULT_IV, key, DEFAULT_SALT, DEFAULT_ITERATIONS, INITIAL_PADDING)
         self.have_more = protocol.source_has_more
         if conn.timeout>0:
             self.timeout_add((conn.timeout + EXTRA_TIMEOUT) * 1000, self.verify_connected)
@@ -402,14 +438,16 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
         mid = get_machine_id()
         if mid:
             capabilities["machine_id"] = mid
-        if self.encryption:
-            assert self.encryption in ENCRYPTION_CIPHERS
+        encryption = self.get_encryption()
+        if encryption:
+            crypto_backend_init()
+            assert encryption in ENCRYPTION_CIPHERS, "invalid encryption '%s', options: %s" % (encryption, csv(ENCRYPTION_CIPHERS))
             iv = get_iv()
             key_salt = get_salt()
             iterations = get_iterations()
             padding = choose_padding(self.server_padding_options)
             up("cipher", {
-                    ""                      : self.encryption,
+                    ""                      : encryption,
                     "iv"                    : iv,
                     "key_salt"              : key_salt,
                     "key_stretch_iterations": iterations,
@@ -417,10 +455,7 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
                     "padding.options"       : PADDING_OPTIONS,
                     })
             key = self.get_encryption_key()
-            if key is None:
-                self.warn_and_quit(EXIT_ENCRYPTION, "encryption key is missing")
-                return None
-            self._protocol.set_cipher_in(self.encryption, iv, key, key_salt, iterations, padding)
+            self._protocol.set_cipher_in(encryption, iv, key, key_salt, iterations, padding)
             netlog("encryption capabilities: %s", dict((k,v) for k,v in capabilities.items() if k.startswith("cipher")))
         capabilities.update(self.hello_extra)
         return capabilities
@@ -441,16 +476,16 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
         return capabilities
 
     def compressed_wrapper(self, datatype, data, level=5):
-        #ugly assumptions here, should pass by name
-        zlib = "zlib" in self.server_compressors and compression.use_zlib
-        lz4 = "lz4" in self.server_compressors and compression.use_lz4
-        lzo = "lzo" in self.server_compressors and compression.use_lzo
-        brotli = False
-        #never use brotli as a generic compressor
-        #brotli = "brotli" in self.server_compressors and compression.use_brotli
-        if level>0 and len(data)>=256 and (zlib or lz4 or lzo):
+        if level>0 and len(data)>=256:
+            #ugly assumptions here, should pass by name
+            zlib = "zlib" in self.server_compressors
+            lz4 = "lz4" in self.server_compressors
+            lzo = "lzo" in self.server_compressors
+            #never use brotli as a generic compressor
+            #brotli = "brotli" in self.server_compressors and compression.use_brotli
             cw = compression.compressed_wrapper(datatype, data, level=level,
-                                                zlib=zlib, lz4=lz4, lzo=lzo, brotli=brotli,
+                                                zlib=zlib, lz4=lz4, lzo=lzo,
+                                                brotli=False, none=True,
                                                 can_inline=False)
             if len(cw)<len(data):
                 #the compressed version is smaller, use it:
@@ -524,7 +559,21 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
         return packet, None, None, None, synchronous, has_more
 
 
+    def stop_progress_process(self):
+        pp = self.progress_process
+        if not pp:
+            return
+        self.progress_process = None
+        if pp.poll():
+            return
+        try:
+            pp.terminate()
+        except Exception:
+            pass
+
     def cleanup(self):
+        self.cancel_progress_timer()
+        self.stop_progress_process()
         reaper_cleanup()
         try:
             FilePrintMixin.cleanup(self)
@@ -547,6 +596,9 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
         register_SIGUSR_signals(GLib.idle_add)
 
     def run(self):
+        self.start_protocol()
+
+    def start_protocol(self):
         #protocol may be None in "listen" mode
         if self._protocol:
             self._protocol.start()
@@ -656,6 +708,20 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
             authlog("password read from tty via getpass: %s", obsc(password))
             self.send_challenge_reply(packet, password)
             return True
+        else:
+            from xpra.platform.paths import get_nodock_command
+            cmd = get_nodock_command()+["_pass", prompt]
+            try:
+                from subprocess import Popen, PIPE
+                proc = Popen(cmd, stdout=PIPE)
+                getChildReaper().add_process(proc, "password-prompt", cmd, True, True)
+                out, err = proc.communicate(None, 60)
+                authlog("err(%s)=%s", cmd, err)
+                password = out.decode()
+                self.send_challenge_reply(packet, password)
+                return True
+            except Exception:
+                log("Error: failed to show GUi for password prompt", exc_info=True)
         return False
 
     def auth_error(self, code, message, server_message="authentication failed"):
@@ -709,13 +775,11 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
                 self.auth_error(EXIT_PASSWORD_REQUIRED,
                                 "this server requires authentication and no password is available")
             return
-        if self.encryption:
+        encryption = self.get_encryption()
+        if encryption:
             assert len(packet)>=3, "challenge does not contain encryption details to use for the response"
             server_cipher = typedict(packet[2])
             key = self.get_encryption_key()
-            if key is None:
-                self.auth_error(EXIT_ENCRYPTION, "the server does not use any encryption", "client requires encryption")
-                return
             if not self.set_server_encryption(server_cipher, key):
                 return
         #all server versions support a client salt,
@@ -764,7 +828,7 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
         self.server_padding_options = caps.strtupleget("cipher.padding.options", (DEFAULT_PADDING,))
         if not cipher or not cipher_iv:
             self.warn_and_quit(EXIT_ENCRYPTION,
-                               "the server does not use or support encryption/password, cannot continue with %s cipher" % self.encryption)
+                               "the server does not use or support encryption/password, cannot continue")
             return False
         if cipher not in ENCRYPTION_CIPHERS:
             self.warn_and_quit(EXIT_ENCRYPTION,
@@ -781,21 +845,42 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
         return True
 
 
+    def get_encryption(self):
+        p = self._protocol
+        if not p:
+            return None
+        conn = p._conn
+        #prefer the socket option, fallback to "--encryption=" option:
+        encryption = conn.options.get("encryption", self.encryption)
+        cryptolog("get_encryption() connection options encryption=%s", encryption)
+        #specifying keyfile or keydata is enough:
+        if not encryption and any(conn.options.get(x) for x in ("encryption-keyfile", "keyfile", "keydata")):
+            cryptolog("found keyfile or keydata attribute, enabling 'AES' encryption")
+            encryption = "AES"
+        if not encryption and os.environ.get("XPRA_ENCRYPTION_KEY"):
+            cryptolog("found encryption key environment variable, enabling 'AES' encryption")
+            encryption = "AES"
+        return encryption
+
     def get_encryption_key(self):
-        key = None
-        if self.encryption_keyfile and os.path.exists(self.encryption_keyfile):
-            key = load_binary_file(self.encryption_keyfile)
-            cryptolog("get_encryption_key() loaded %i bytes from '%s'", len(key or ""), self.encryption_keyfile)
-        else:
-            cryptolog("get_encryption_key() file '%s' does not exist", self.encryption_keyfile)
-        if not key:
-            XPRA_ENCRYPTION_KEY = "XPRA_ENCRYPTION_KEY"
-            key = strtobytes(os.environ.get(XPRA_ENCRYPTION_KEY, ''))
-            cryptolog("get_encryption_key() got %i bytes from '%s' environment variable",
-                      len(key or ""), XPRA_ENCRYPTION_KEY)
-        if not key:
-            raise InitExit(1, "no encryption key")
-        return key.strip(b"\n\r")
+        conn = self._protocol._conn
+        keydata = parse_encoded_bin_data(conn.options.get("keydata", None))
+        cryptolog("get_encryption_key() connection options keydata=%s", ellipsizer(keydata))
+        keyfile = conn.options.get("encryption-keyfile") or conn.options.get("keyfile") or self.encryption_keyfile
+        if keyfile:
+            if os.path.exists(keyfile):
+                key = filedata_nocrlf(keyfile)
+                cryptolog("get_encryption_key() loaded %i bytes from '%s'",
+                          len(key or ""), keyfile)
+                return key
+            cryptolog("get_encryption_key() file '%s' does not exist", keyfile)
+        XPRA_ENCRYPTION_KEY = "XPRA_ENCRYPTION_KEY"
+        key = strtobytes(os.environ.get(XPRA_ENCRYPTION_KEY, ''))
+        cryptolog("get_encryption_key() got %i bytes from '%s' environment variable",
+                  len(key or ""), XPRA_ENCRYPTION_KEY)
+        if key:
+            return key.strip(b"\n\r")
+        raise InitExit(1, "no encryption key")
 
     def _process_hello(self, packet):
         self.remove_packet_handlers("challenge")
@@ -825,9 +910,6 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
         if not self.parse_network_capabilities(caps):
             netlog("server_connection_established(..) failed network capabilities")
             return False
-        #raise packet size if required:
-        if self.file_transfer:
-            self._protocol.max_packet_size = max(self._protocol.max_packet_size, self.file_size_limit*1024*1024)
         netlog("server_connection_established(..) adding authenticated packet handlers")
         self.init_authenticated_packet_handlers()
         return True
@@ -854,7 +936,8 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
         p = self._protocol
         if not p:
             return False
-        if self.encryption:
+        encryption = self.get_encryption()
+        if encryption:
             #server uses a new cipher after second hello:
             key = self.get_encryption_key()
             assert key, "encryption key is missing"
@@ -876,7 +959,7 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
         log("process_gibberish(%s)", ellipsizer(packet))
         message, data = packet[1:3]
         p = self._protocol
-        show_as_text = p and p.input_packetcount==0 and all(c in string.printable for c in bytestostr(data))
+        show_as_text = p and p.input_packetcount==0 and len(data)<128 and all(c in string.printable for c in bytestostr(data))
         if show_as_text:
             #looks like the first packet back is just text, print it:
             data = bytestostr(data)
@@ -887,7 +970,14 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
                 netlog.error("Error: failed to connect, received")
                 netlog.error(" %s", repr_ellipsized(data))
         else:
-            netlog.error("Error: received uninterpretable nonsense: %s", message)
+            from xpra.net.socket_util import guess_packet_type
+            packet_type = guess_packet_type(data)
+            log("guess_packet_type(%r)=%s", data, packet_type)
+            if packet_type and packet_type!="xpra":
+                netlog.error("Error: received a %s packet", packet_type)
+                netlog.error(" this is not an xpra server?")
+            else:
+                netlog.error("Error: received uninterpretable nonsense: %s", message)
             if p:
                 netlog.error(" packet no %i data: %s", p.input_packetcount, repr_ellipsized(data))
             else:
@@ -906,10 +996,7 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
     def remove_packet_handlers(self, *keys):
         for k in keys:
             for d in (self._packet_handlers, self._ui_packet_handlers):
-                try:
-                    del d[k]
-                except KeyError:
-                    pass
+                d.pop(k, None)
 
     def init_packet_handlers(self):
         self._packet_handlers = {}
