@@ -5,7 +5,7 @@
 
 from xpra.platform import program_context
 from xpra.platform.gui import force_focus
-from xpra.gtk_common.gtk_util import add_close_accel
+from xpra.gtk_common.gtk_util import add_close_accel, get_icon_pixbuf
 
 import cairo
 import gi
@@ -21,6 +21,9 @@ class TransparentColorWindow(Gtk.Window):
         self.set_title("Transparent Colors")
         self.set_position(Gtk.WindowPosition.CENTER)
         self.set_default_size(320, 320)
+        icon = get_icon_pixbuf("encoding.png")
+        if icon:
+            self.set_icon(icon)
         screen = self.get_screen()
         visual = screen.get_rgba_visual()
         if visual and screen.is_composited():
@@ -90,10 +93,10 @@ def main():
         set_default_icon("encoding.png")
         init()
 
-        import signal
+        from xpra.gtk_common.gobject_compat import register_os_signals
         def signal_handler(*_args):
             Gtk.main_quit()
-        signal.signal(signal.SIGINT, signal_handler)
+        register_os_signals(signal_handler)
         w = TransparentColorWindow()
         add_close_accel(w, Gtk.main_quit)
         GLib.idle_add(w.show_with_focus)
