@@ -6,19 +6,25 @@
 
 import os
 import re
-from gi.repository import GLib, Gtk
+from gi.repository import GLib, Gtk, GdkPixbuf
 
-from xpra.util import CLIENT_EXIT, iround, envbool, repr_ellipsized, reverse_dict, typedict
+from xpra.util import (
+    CLIENT_EXIT,
+    iround, envbool,
+    ellipsizer, repr_ellipsized, reverse_dict, typedict,
+    )
 from xpra.os_util import bytestostr, OSX, WIN32
 from xpra.gtk_common.gtk_util import (
-    menuitem, get_pixbuf_from_data, scaled_image,
+    get_pixbuf_from_data, scaled_image,
+    )
+from xpra.client.gtk_base.menu_helper import (
+    MenuHelper,
+    ll, set_sensitive, ensure_item_selected,
+    make_encodingsmenu, make_min_auto_menu,
     )
 from xpra.client.client_base import EXIT_OK
-from xpra.gtk_common.about import about, close_about
-from xpra.codecs.codec_constants import PREFERED_ENCODING_ORDER
-from xpra.codecs.loader import get_encoding_help, get_encoding_name
+from xpra.codecs.codec_constants import PREFERRED_ENCODING_ORDER
 from xpra.simple_stats import std_unit_dec
-from xpra.platform.gui import get_icon_size
 from xpra.platform.paths import get_icon_dir
 from xpra.client import mixin_features
 from xpra.log import Logger
@@ -30,10 +36,9 @@ webcamlog = Logger("menu", "webcam")
 avsynclog = Logger("menu", "av-sync")
 bandwidthlog = Logger("bandwidth", "network")
 
-HIDE_DISABLED_MENU_ENTRIES = OSX
-
 SHOW_TITLE_ITEM = envbool("XPRA_SHOW_TITLE_ITEM", True)
 SHOW_VERSION_CHECK = envbool("XPRA_SHOW_VERSION_CHECK", True)
+SHOW_QR = envbool("XPRA_SHOW_QR", True)
 SHOW_UPLOAD = envbool("XPRA_SHOW_UPLOAD_MENU", True)
 SHOW_SERVER_LOG = envbool("XPRA_SHOW_SERVER_LOG", True)
 SHOW_DOWNLOAD = envbool("XPRA_SHOW_DOWNLOAD", True)
@@ -104,229 +109,13 @@ CLIPBOARD_DIRECTION_LABEL_TO_NAME = {
 CLIPBOARD_DIRECTION_NAME_TO_LABEL = reverse_dict(CLIPBOARD_DIRECTION_LABEL_TO_NAME)
 
 
-def ll(m):
-    try:
-        return "%s:%s" % (type(m), m.get_label())
-    except AttributeError:
-        return str(m)
+class GTKTrayMenuBase(MenuHelper):
 
-def set_sensitive(widget, sensitive):
-    if OSX:
-        if sensitive:
-            widget.show()
-        else:
-            widget.hide()
-    widget.set_sensitive(sensitive)
+    def setup_menu(self):
+        return self.do_setup_menu(SHOW_CLOSE)
 
-
-#utility method to ensure there is always only one CheckMenuItem
-#selected in a submenu:
-def ensure_item_selected(submenu, item, recurse=True):
-    if not isinstance(item, Gtk.CheckMenuItem):
-        return None
-    if item.get_active():
-        #deactivate all except this one
-        def deactivate(items, skip=None):
-            for x in items:
-                if x==skip:
-                    continue
-                if isinstance(x, Gtk.MenuItem):
-                    submenu = x.get_submenu()
-                    if submenu and recurse:
-                        deactivate(submenu.get_children(), skip)
-                if isinstance(x, Gtk.CheckMenuItem):
-                    if x!=item and x.get_active():
-                        x.set_active(False)
-        deactivate(submenu.get_children(), item)
-        return item
-    #ensure there is at least one other active item
-    def get_active_item(items):
-        for x in items:
-            if isinstance(x, Gtk.MenuItem):
-                submenu = x.get_submenu()
-                if submenu:
-                    a = get_active_item(submenu.get_children())
-                    if a:
-                        return a
-            if isinstance(x, Gtk.CheckMenuItem):
-                if x.get_active():
-                    return x
-        return None
-    active = get_active_item(submenu.get_children())
-    if active:
-        return active
-    #if not then keep this one active:
-    item.set_active(True)
-    return item
-
-
-def make_min_auto_menu(title, min_options, options,
-                       get_current_min_value,
-                       get_current_value,
-                       set_min_value_cb,
-                       set_value_cb):
-    #note: we must keep references to the parameters on the submenu
-    #(closures and gtk callbacks don't mix so well!)
-    submenu = Gtk.Menu()
-    submenu.get_current_min_value = get_current_min_value
-    submenu.get_current_value = get_current_value
-    submenu.set_min_value_cb = set_min_value_cb
-    submenu.set_value_cb = set_value_cb
-    fstitle = Gtk.MenuItem()
-    fstitle.set_label("Fixed %s:" % title)
-    set_sensitive(fstitle, False)
-    submenu.append(fstitle)
-    submenu.menu_items = {}
-    submenu.min_menu_items = {}
-    def populate_menu(options, value, set_fn):
-        found_match = False
-        items = {}
-        if value and value>0 and value not in options:
-            options[value] = "%s%%" % value
-        for s in sorted(options.keys()):
-            t = options.get(s)
-            qi = Gtk.CheckMenuItem(label=t)
-            qi.set_draw_as_radio(True)
-            candidate_match = s>=max(0, value)
-            qi.set_active(not found_match and candidate_match)
-            found_match |= candidate_match
-            qi.connect('activate', set_fn, submenu)
-            if s>0:
-                qi.set_tooltip_text("%s%%" % s)
-            submenu.append(qi)
-            items[s] = qi
-        return items
-    def set_value(item, ss):
-        if not item.get_active():
-            return
-        #user select a new value from the menu:
-        s = -1
-        for ts,tl in options.items():
-            if tl==item.get_label():
-                s = ts
-                break
-        if s>=0 and s!=ss.get_current_value():
-            log("setting %s to %s", title, s)
-            ss.set_value_cb(s)
-            #deselect other items:
-            for x in ss.menu_items.values():
-                if x!=item:
-                    x.set_active(False)
-            #min is only relevant in auto-mode:
-            if s!=0:
-                for v,x in ss.min_menu_items.items():
-                    x.set_active(v==0)
-    submenu.menu_items.update(populate_menu(options, get_current_value(), set_value))
-    submenu.append(Gtk.SeparatorMenuItem())
-    mstitle = Gtk.MenuItem()
-    mstitle.set_label("Minimum %s:" % title)
-    set_sensitive(mstitle, False)
-    submenu.append(mstitle)
-    def set_min_value(item, ss):
-        if not item.get_active():
-            return
-        #user selected a new min-value from the menu:
-        s = -1
-        for ts,tl in min_options.items():
-            if tl==item.get_label():
-                s = ts
-                break
-        if s>=0 and s!=ss.get_current_min_value():
-            log("setting min-%s to %s", title, s)
-            ss.set_min_value_cb(s)
-            #deselect other min items:
-            for x in ss.min_menu_items.values():
-                if x!=item:
-                    x.set_active(False)
-            #min requires auto-mode:
-            for x in ss.menu_items.values():
-                if x.get_label()=="Auto":
-                    if not x.get_active():
-                        x.activate()
-                else:
-                    x.set_active(False)
-    mv = -1
-    if get_current_value()<=0:
-        mv = get_current_min_value()
-    submenu.min_menu_items.update(populate_menu(min_options, mv, set_min_value))
-    submenu.show_all()
-    return submenu
-
-def make_encodingsmenu(get_current_encoding, set_encoding, encodings, server_encodings):
-    encodings_submenu = Gtk.Menu()
-    populate_encodingsmenu(encodings_submenu, get_current_encoding, set_encoding, encodings, server_encodings)
-    return encodings_submenu
-
-def populate_encodingsmenu(encodings_submenu, get_current_encoding, set_encoding, encodings, server_encodings):
-    encodings_submenu.get_current_encoding = get_current_encoding
-    encodings_submenu.set_encoding = set_encoding
-    encodings_submenu.encodings = encodings
-    encodings_submenu.server_encodings = server_encodings
-    encodings_submenu.index_to_encoding = {}
-    encodings_submenu.encoding_to_index = {}
-    NAME_TO_ENCODING = {}
-    for i, encoding in enumerate(encodings):
-        name = get_encoding_name(encoding)
-        descr = get_encoding_help(encoding)
-        NAME_TO_ENCODING[name] = encoding
-        encoding_item = Gtk.CheckMenuItem(label=name)
-        if descr:
-            if encoding not in server_encodings:
-                descr += "\n(not available on this server)"
-            encoding_item.set_tooltip_text(descr)
-        def encoding_changed(item):
-            ensure_item_selected(encodings_submenu, item)
-            enc = NAME_TO_ENCODING.get(item.get_label())
-            log("encoding_changed(%s) enc=%s, current=%s", item, enc, encodings_submenu.get_current_encoding())
-            if enc is not None and encodings_submenu.get_current_encoding()!=enc:
-                encodings_submenu.set_encoding(enc)
-        log("populate_encodingsmenu(..) encoding=%s, current=%s, active=%s",
-            encoding, get_current_encoding(), encoding==get_current_encoding())
-        encoding_item.set_active(encoding==get_current_encoding())
-        sensitive = encoding in server_encodings
-        if not sensitive and HIDE_DISABLED_MENU_ENTRIES:
-            continue
-        set_sensitive(encoding_item, encoding in server_encodings)
-        encoding_item.set_draw_as_radio(True)
-        encoding_item.connect("toggled", encoding_changed)
-        encodings_submenu.append(encoding_item)
-        encodings_submenu.index_to_encoding[i] = encoding
-        encodings_submenu.encoding_to_index[encoding] = i
-    encodings_submenu.show_all()
-
-
-class GTKTrayMenuBase:
-
-    def __init__(self, client):
-        self.client = client
-        self.menu = None
-        self.menu_shown = False
-        self.menu_icon_size = 0
-
-    def build(self):
-        if self.menu is None:
-            try:
-                self.menu = self.setup_menu(SHOW_CLOSE)
-            except Exception as e:
-                log("build()", exc_info=True)
-                log.error("Error: failed to setup menu")
-                log.error(" %s", e)
-        return self.menu
-
-    def show_session_info(self, *args):
-        self.client.show_session_info(*args)
-
-    def show_bug_report(self, *args):
-        self.client.show_bug_report(*args)
-
-
-    def get_image(self, icon_name, size=None):
-        return self.client.get_image(icon_name, size)
-
-    def setup_menu(self, show_close=True):
+    def do_setup_menu(self, show_close):
         log("setup_menu(%s)", show_close)
-        self.menu_shown = False
-        self.menu_icon_size = get_icon_size()
         menu = Gtk.Menu()
         title_item = None
         if SHOW_TITLE_ITEM:
@@ -346,7 +135,7 @@ class GTKTrayMenuBase:
         menu.append(self.make_infomenuitem())
         menu.append(self.make_featuresmenuitem())
         if mixin_features.windows and self.client.keyboard_helper:
-            menu.append(self.make_layoutsmenuitem())
+            menu.append(self.make_keyboardmenuitem())
         if mixin_features.clipboard and SHOW_CLIPBOARD_MENU:
             menu.append(self.make_clipboardmenuitem())
         if mixin_features.windows:
@@ -369,62 +158,6 @@ class GTKTrayMenuBase:
         log("setup_menu(%s) done", show_close)
         return menu
 
-    def cleanup(self):
-        self.close_menu()
-        close_about()
-
-    def close_menu(self, *_args):
-        if self.menu_shown:
-            self.menu.popdown()
-            self.menu_shown = False
-
-    def menu_deactivated(self, *_args):
-        self.menu_shown = False
-
-    def activate(self, button=1, time=0):
-        log("activate(%s, %s)", button, time)
-        self.show_menu(button, time)
-
-    def popup(self, button, time):
-        log("popup(%s, %s)", button, time)
-        self.show_menu(button, time)
-
-    def show_menu(self, _button, _time):
-        raise NotImplementedError("override me!")
-
-
-    def handshake_menuitem(self, *args, **kwargs):
-        """ Same as menuitem() but this one will be disabled until we complete the server handshake """
-        mi = self.menuitem(*args, **kwargs)
-        set_sensitive(mi, False)
-        def enable_menuitem(*_args):
-            set_sensitive(mi, True)
-        self.client.after_handshake(enable_menuitem)
-        return mi
-
-
-    def make_menu(self):
-        return Gtk.Menu()
-
-    def menuitem(self, title, icon_name=None, tooltip=None, cb=None, **kwargs):
-        """ Utility method for easily creating an ImageMenuItem """
-        image = None
-        if MENU_ICONS:
-            image = kwargs.get("image")
-            if icon_name and not image:
-                icon_size = self.menu_icon_size or get_icon_size()
-                image = self.get_image(icon_name, icon_size)
-        return menuitem(title, image, tooltip, cb)
-
-    def checkitem(self, title, cb=None, active=False):
-        """ Utility method for easily creating a CheckMenuItem """
-        check_item = Gtk.CheckMenuItem(label=title)
-        check_item.set_active(active)
-        if cb:
-            check_item.connect("toggled", cb)
-        check_item.show()
-        return check_item
-
 
     def make_infomenuitem(self):
         info_menu_item = self.menuitem("Information", "information.png")
@@ -432,37 +165,14 @@ class GTKTrayMenuBase:
         info_menu_item.set_submenu(menu)
         menu.append(self.make_aboutmenuitem())
         menu.append(self.make_sessioninfomenuitem())
+        if SHOW_QR:
+            menu.append(self.make_qrmenuitem())
         if SHOW_VERSION_CHECK:
             menu.append(self.make_updatecheckmenuitem())
         menu.append(self.make_bugreportmenuitem())
         info_menu_item.show_all()
         return info_menu_item
 
-    def make_aboutmenuitem(self):
-        return self.menuitem("About Xpra", "xpra.png", None, about)
-
-    def make_updatecheckmenuitem(self):
-        def show_update_window(*_args):
-            from xpra.client.gtk_base.update_status import getUpdateStatusWindow
-            w = getUpdateStatusWindow()
-            w.show()
-            w.check()
-        return self.menuitem("Check for updates", "update.png", None, show_update_window)
-
-    def make_sessioninfomenuitem(self):
-        def show_session_info_cb(*_args):
-            #we define a generic callback to remove the arguments
-            #(which contain the menu widget and are of no interest to the 'show_session_info' function)
-            self.show_session_info()
-        sessioninfomenuitem = self.handshake_menuitem("Session Info", "statistics.png", None, show_session_info_cb)
-        return sessioninfomenuitem
-
-    def make_bugreportmenuitem(self):
-        def show_bug_report_cb(*_args):
-            self.show_bug_report()
-        return  self.menuitem("Bug Report", "bugs.png", None, show_bug_report_cb)
-
-
     def make_featuresmenuitem(self):
         features_menu_item = self.handshake_menuitem("Features", "features.png")
         menu = Gtk.Menu()
@@ -485,8 +195,6 @@ class GTKTrayMenuBase:
             menu.append(self.make_openglmenuitem())
         if mixin_features.windows:
             menu.append(self.make_modalwindowmenuitem())
-        if mixin_features.windows and self.client.keyboard_helper:
-            menu.append(self.make_keyboardsyncmenuitem())
 
     def make_sharingmenuitem(self):
         def sharing_toggled(*args):
@@ -720,13 +428,12 @@ class GTKTrayMenuBase:
             if not c.server_clipboard:
                 self.clipboard_menuitem.set_tooltip_text("Server does not support clipboard synchronization")
                 return
-            if not c.client_supports_clipboard:
+            ch = c.clipboard_helper
+            if not c.client_supports_clipboard or not ch:
                 self.clipboard_menuitem.set_tooltip_text("Client does not support clipboard synchronization")
                 return
             #add a submenu:
             set_sensitive(self.clipboard_menuitem, True)
-            c = self.client
-            ch = self.client.clipboard_helper
             clipboard_submenu = Gtk.Menu()
             self.clipboard_menuitem.set_submenu(clipboard_submenu)
             if WIN32 or OSX:
@@ -772,7 +479,7 @@ class GTKTrayMenuBase:
                 log("keyboard_sync_toggled(%s) keyboard_sync=%s", args, ks)
                 set_keyboard_sync_tooltip()
                 self.client.send_keyboard_sync_enabled_status()
-        self.keyboard_sync_menuitem = self.checkitem("Keyboard Synchronization")
+        self.keyboard_sync_menuitem = self.checkitem("State Synchronization")
         set_sensitive(self.keyboard_sync_menuitem, False)
         def set_keyboard_sync_menuitem(*args):
             kh = self.client.keyboard_helper
@@ -787,6 +494,23 @@ class GTKTrayMenuBase:
         self.client.after_handshake(set_keyboard_sync_menuitem)
         return self.keyboard_sync_menuitem
 
+    def make_shortcutsmenuitem(self):
+        self.keyboard_shortcuts_menuitem = self.checkitem("Intercept Shortcuts")
+        kh = self.client.keyboard_helper
+        self.keyboard_shortcuts_menuitem.set_active(kh and bool(kh.shortcuts_enabled))
+        def keyboard_shortcuts_toggled(*args):
+            ks = self.keyboard_shortcuts_menuitem.get_active()
+            log("keyboard_shortcuts_toggled%s enabled=%s", args, ks)
+            kh.shortcuts_enabled = ks
+        self.keyboard_shortcuts_menuitem.connect("toggled", keyboard_shortcuts_toggled)
+        return self.keyboard_shortcuts_menuitem
+
+    def make_viewshortcutsmenuitem(self):
+        def show_shortcuts(*_args):
+            self.client.show_shortcuts()
+        return self.menuitem("View Shortcuts", tooltip="Show all active keyboard shortcuts", cb=show_shortcuts)
+
+
     def make_openglmenuitem(self):
         gl = self.checkitem("OpenGL")
         gl.set_tooltip_text("hardware accelerated rendering using OpenGL")
@@ -932,7 +656,7 @@ class GTKTrayMenuBase:
 
     def make_encodingssubmenu(self):
         server_encodings = list(self.client.server_encodings)
-        all_encodings = [x for x in PREFERED_ENCODING_ORDER if x in self.client.get_encodings()]
+        all_encodings = [x for x in PREFERRED_ENCODING_ORDER if x in self.client.get_encodings()]
         encodings = [x for x in all_encodings if x not in self.client.server_encodings_problematic]
         if not encodings:
             #all we have, show the "bad" hidden ones then!
@@ -1355,8 +1079,20 @@ class GTKTrayMenuBase:
         self.client.on_server_setting_changed("webcam", webcam_changed)
         return webcam
 
+
+    def make_keyboardmenuitem(self):
+        keyboard_menu_item = self.handshake_menuitem("Keyboard", "keyboard.png")
+        menu = Gtk.Menu()
+        keyboard_menu_item.set_submenu(menu)
+        menu.append(self.make_keyboardsyncmenuitem())
+        menu.append(self.make_shortcutsmenuitem())
+        menu.append(self.make_viewshortcutsmenuitem())
+        menu.append(self.make_layoutsmenuitem())
+        keyboard_menu_item.show_all()
+        return keyboard_menu_item
+
     def make_layoutsmenuitem(self):
-        keyboard = self.menuitem("Keyboard", "keyboard.png", "Select your keyboard layout", None)
+        keyboard = self.menuitem("Layout", "keyboard.png", "Select your keyboard layout", None)
         set_sensitive(keyboard, False)
         self.layout_submenu = Gtk.Menu()
         keyboard.set_submenu(self.layout_submenu)
@@ -1394,6 +1130,8 @@ class GTKTrayMenuBase:
         layouts = tuple(bytestostr(x) for x in layouts)
         variant = bytestostr(variant or b"")
         variants = tuple(bytestostr(x) for x in variants)
+        log("make_layoutsmenuitem() layout=%s, layouts=%s, variant=%s, variants=%s",
+            layout, layouts, variant, variants)
         full_layout_list = False
         if len(layouts)>1:
             log("keyboard layouts: %s", ",".join(bytestostr(x) for x in layouts))
@@ -1441,7 +1179,11 @@ class GTKTrayMenuBase:
                     #no variants:
                     self.layout_submenu.append(kbitem(name, layout, None))
         keyboard_helper = self.client.keyboard_helper
-        def set_layout_enabled(*_args):
+        def set_layout_enabled(*args):
+            log("set_layout_enabled%s full_layout_list=%s",
+                     args, full_layout_list)
+            log("set_layout_enabled%s layout=%s, print=%s, query=%s",
+                     args, keyboard_helper.xkbmap_layout, keyboard_helper.xkbmap_print, keyboard_helper.xkbmap_query)
             if full_layout_list and \
             (keyboard_helper.xkbmap_layout or keyboard_helper.xkbmap_print or keyboard_helper.xkbmap_query):
                 #we have detected a layout
@@ -1587,12 +1329,16 @@ class GTKTrayMenuBase:
 
 
     def make_serverlogmenuitem(self):
-        self.download_log = self.menuitem("Download Server Log", "list.png", cb=self.client.download_server_log)
+        def download_server_log(*_args):
+            self.client.download_server_log()
+        self.download_log = self.menuitem("Download Server Log", "list.png", cb=download_server_log)
         def enable_download(*args):
             log("enable_download%s server_file_transfer=%s", args, self.client.remote_file_transfer)
-            set_sensitive(self.download_log, self.client.remote_file_transfer)
+            set_sensitive(self.download_log, self.client.remote_file_transfer and bool(self.client._remote_server_log))
             if not self.client.remote_file_transfer:
                 self.download_log.set_tooltip_text("Not supported by the server")
+            elif not self.client._remote_server_log:
+                self.download_log.set_tooltip_text("Server does not expose its log-file")
             else:
                 self.download_log.set_tooltip_text("Download the server log")
         self.client.after_handshake(enable_download)
@@ -1601,10 +1347,19 @@ class GTKTrayMenuBase:
 
     def make_shutdownmenuitem(self):
         def ask_shutdown_confirm(*_args):
+            messages = []
+            #uri = self.client.display_desc.get("display_name")
+            #if uri:
+            #    messages.append("URI: %s" % uri)
+            session_name = self.client.session_name or self.client.server_session_name
+            if session_name:
+                messages.append("Shutting down the session '%s' may result in data loss," % session_name)
+            else:
+                messages.append("Shutting down this session may result in data loss,")
+            messages.append("are you sure you want to proceed?")
             dialog = Gtk.MessageDialog (None, 0, Gtk.MessageType.QUESTION,
                                     Gtk.ButtonsType.NONE,
-                                    "Shutting down this session may cause data loss,\n"+
-                                    "are you sure you want to proceed?")
+                                    "\n".join(messages))
             dialog.add_button(Gtk.STOCK_CANCEL, 0)
             SHUTDOWN = 1
             dialog.add_button("Shutdown", SHUTDOWN)
@@ -1652,7 +1407,7 @@ class GTKTrayMenuBase:
     def build_start_menu(self):
         menu = Gtk.Menu()
         execlog("build_start_menu() %i menu items", len(self.client.server_xdg_menu))
-        execlog("self.client.server_xdg_menu=%s", self.client.server_xdg_menu)
+        execlog("self.client.server_xdg_menu=%s", ellipsizer(self.client.server_xdg_menu))
         for cat, category_props in sorted(self.client.server_xdg_menu.items()):
             category = cat.decode("utf-8")
             execlog(" * category: %s", category)
@@ -1661,7 +1416,7 @@ class GTKTrayMenuBase:
                 execlog("category properties is not a dict: %s", type(category_props))
                 continue
             cp = typedict(category_props)
-            execlog("  category_props(%s)=%s", category, category_props)
+            execlog("  category_props(%s)=%s", category, ellipsizer(category_props))
             entries = cp.dictget("Entries")
             if not entries:
                 execlog("  no entries for category '%s'", category)
@@ -1690,13 +1445,14 @@ class GTKTrayMenuBase:
         if not pixbuf and icondata:
             #gtk pixbuf loader:
             try:
-                from gi.repository import GdkPixbuf
                 loader = GdkPixbuf.PixbufLoader()
                 loader.write(icondata)
                 loader.close()
                 pixbuf = loader.get_pixbuf()
-            except Exception:
-                log.error("Error: failed to load icon data for %s", bytestostr(app_name), exc_info=True)
+            except Exception as e:
+                log("pixbuf loader failed", exc_info=True)
+                log.error("Error: failed to load icon data for '%s':", bytestostr(app_name))
+                log.error(" %s", e)
                 log.error(" data=%s", repr_ellipsized(icondata))
         if not pixbuf and icondata:
             #let's try pillow:
