@@ -13,6 +13,7 @@ from xpra.platform.gui import (
     )
 from xpra.scripts.main import check_display
 from xpra.scripts.config import FALSE_OPTIONS
+from xpra.net.common import MAX_PACKET_SIZE
 from xpra.os_util import monotonic_time
 from xpra.util import (
     iround, envint, envfloat, envbool, log_screen_sizes, engs, flatten_dict, typedict,
@@ -248,7 +249,7 @@ class DisplayClient(StubClientMixin):
             if self.server_is_desktop and not self.desktop_fullscreen:
                 #don't honour auto mode in this case
                 if self.desktop_scaling=="auto":
-                    log.info(" not scaling a shadow server")
+                    log.info(" not scaling a %s server", c.strget("type", "shadow"))
                     skip_vfb_size_check = self.xscale>1 or self.yscale>1
                     self.scale_change_embargo = 0
                     self.scalingoff()
@@ -286,16 +287,14 @@ class DisplayClient(StubClientMixin):
             raise SystemExit(message)
         if maxw>=16384 or maxh>=16384:
             log.warn("Warning: the desktop size is extremely large: %ix%i", maxw, maxh)
-        #max packet size to accomodate:
-        # * full screen RGBX (32 bits) uncompressed
-        # * file-size-limit
-        # both with enough headroom for some metadata (4k)
+        #max packet size to accomodate
+        # a full screen RGBX (32 bits) uncompressed image
+        # also with enough headroom for some metadata (4k)
         p = self._protocol
         if p:
             #we can't assume to have a real ClientConnection object:
-            file_size_limit = getattr(self, "file_size_limit", 0)
-            p.max_packet_size = max(maxw*maxh*4, file_size_limit*1024*1024) + 4*1024
-            p.abs_max_packet_size = max(maxw*maxh*4 * 4, file_size_limit*1024*1024) + 4*1024
+            p.max_packet_size = max(MAX_PACKET_SIZE, maxw*maxh*4 + 4*1024)
+            p.abs_max_packet_size = maxw*maxh*4*4 + 4*1024
             log("maximum packet size set to %i", p.max_packet_size)
 
 
