@@ -5,6 +5,7 @@
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+import os
 import sys
 
 from xpra.client.client_base import XpraClientBase
@@ -13,8 +14,14 @@ from xpra.platform import set_name
 from xpra.platform.gui import ready as gui_ready, get_wm_name, get_session_type, ClientExtras
 from xpra.version_util import full_version_str
 from xpra.net import compression, packet_encoding
+from xpra.net.net_util import get_info as get_net_info
 from xpra.child_reaper import reaper_cleanup
-from xpra.os_util import platform_name, bytestostr, strtobytes, BITS, POSIX, is_Wayland
+from xpra.platform.info import get_sys_info
+from xpra.os_util import (
+    platform_name, bytestostr, strtobytes,
+    BITS, POSIX, is_Wayland,
+    get_frame_info, get_info_env, get_sysconfig_info,
+    )
 from xpra.util import (
     std, envbool, envint, typedict, updict, repr_ellipsized, ellipsizer, log_screen_sizes, engs, csv,
     merge_dicts,
@@ -76,6 +83,7 @@ log("UIXpraClient%s: %s", ClientBaseClass, CLIENT_BASES)
 
 NOTIFICATION_EXIT_DELAY = envint("XPRA_NOTIFICATION_EXIT_DELAY", 2)
 MOUSE_DELAY_AUTO = envbool("XPRA_MOUSE_DELAY_AUTO", True)
+SYSCONFIG = envbool("XPRA_SYSCONFIG", False)
 
 
 """
@@ -128,6 +136,7 @@ class UIXpraClient(ClientBaseClass):
         self.server_xdg_menu = None
         self.start_new_commands  = []
         self.start_child_new_commands  = []
+        self.headerbar = None
 
         #in WindowClient - should it be?
         #self.server_is_desktop = False
@@ -169,6 +178,7 @@ class UIXpraClient(ClientBaseClass):
         self.readonly = opts.readonly
         self.client_supports_sharing = opts.sharing is True
         self.client_lock = opts.lock is True
+        self.headerbar = opts.headerbar
 
 
     def init_ui(self, opts):
@@ -183,8 +193,10 @@ class UIXpraClient(ClientBaseClass):
             overrides = [noauto(getattr(opts, "keyboard_%s" % x)) for x in (
                 "layout", "layouts", "variant", "variants", "options",
                 )]
+            def send_keyboard(*parts):
+                self.after_handshake(self.send, *parts)
             try:
-                self.keyboard_helper = self.keyboard_helper_class(self.send, opts.keyboard_sync,
+                self.keyboard_helper = self.keyboard_helper_class(send_keyboard, opts.keyboard_sync,
                                                                   opts.shortcut_modifiers,
                                                                   opts.key_shortcut,
                                                                   opts.keyboard_raw, *overrides)
@@ -207,8 +219,7 @@ class UIXpraClient(ClientBaseClass):
 
         if MOUSE_DELAY_AUTO:
             try:
-                from xpra.platform.gui import get_vrefresh
-                v = get_vrefresh()
+                v = self.get_vrefresh()
                 if v<=0:
                     #some platforms don't detect the vrefresh correctly
                     #(ie: macos in virtualbox?), so use a sane default:
@@ -218,6 +229,11 @@ class UIXpraClient(ClientBaseClass):
             except Exception:
                 log("failed to calculate automatic delay", exc_info=True)
 
+    def get_vrefresh(self):
+        #this method is overriden in the GTK client
+        from xpra.platform.gui import get_vrefresh
+        return get_vrefresh()
+
 
     def run(self):
         if self.client_extras:
@@ -255,7 +271,15 @@ class UIXpraClient(ClientBaseClass):
 
 
     def get_info(self):
-        info = {}
+        info = {
+            "pid"       : os.getpid(),
+            "threads"   : get_frame_info(),
+            "env"       : get_info_env(),
+            "sys"       : get_sys_info(),
+            "network"   : get_net_info(),
+            }
+        if SYSCONFIG:
+            info["sysconfig"] = get_sysconfig_info()
         for c in CLIENT_BASES:
             try:
                 i = c.get_info(self)
@@ -488,7 +512,19 @@ class UIXpraClient(ClientBaseClass):
         if self.tray:
             self.tray.ready()
         self.send_info_request()
-
+        msg = "running"
+        try:
+            windows = tuple(self._id_to_window.values())
+        except AttributeError:
+            pass
+        else:
+            trays = sum(1 for w in windows if w.is_tray())
+            wins = sum(1 for w in windows if not w.is_tray())
+            if wins:
+                msg += ", %i window%s" % (wins, engs(wins))
+            if trays:
+                msg += ", %i tray%s" % (trays, engs(trays))
+        log.info(msg)
 
     def handshake_complete(self):
         oh = self._on_handshake
@@ -566,7 +602,7 @@ class UIXpraClient(ClientBaseClass):
         command = bytestostr(packet[1])
         if command=="show_session_info":
             args = packet[2:]
-            log("calling show_session_info%s on server request", args)
+            log("calling %s%s on server request", self.show_session_info, args)
             self.show_session_info(*args)
         elif command=="show_bug_report":
             self.show_bug_report()
@@ -709,10 +745,10 @@ class UIXpraClient(ClientBaseClass):
 
     def handle_key_action(self, window, key_event):
         if self.readonly or self.keyboard_helper is None:
-            return
+            return False
         wid = self._window_to_id[window]
         keylog("handle_key_action(%s, %s) wid=%s", window, key_event, wid)
-        self.keyboard_helper.handle_key_action(window, wid, key_event)
+        return self.keyboard_helper.handle_key_action(window, wid, key_event)
 
     def mask_to_names(self, mask):
         if self.keyboard_helper is None:
@@ -727,10 +763,7 @@ class UIXpraClient(ClientBaseClass):
         metadata = typedict(metadata)
         if self.server_is_desktop and self.desktop_fullscreen:
             #force it fullscreen:
-            try:
-                del metadata["size-constraints"]
-            except KeyError:
-                pass
+            metadata.pop("size-constraints", None)
             metadata["fullscreen"] = True
             #FIXME: try to figure out the monitors we go fullscreen on for X11:
             #if POSIX:
