@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
-# Copyright (C) 2013-2019 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2013-2020 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -9,7 +9,7 @@
 import socket
 import sys
 
-from xpra.os_util import WIN32, OSX, PYTHON3
+from xpra.os_util import WIN32
 from xpra.log import Logger
 
 log = Logger("network", "util")
@@ -17,7 +17,7 @@ log = Logger("network", "util")
 
 netifaces_version = 0
 _netifaces = None
-def import_netifaces():
+def import_netifaces() -> object:
     global _netifaces, netifaces_version
     if _netifaces is None:
         try:
@@ -34,7 +34,7 @@ iface_ipmasks = {}
 bind_IPs = None
 
 
-def get_free_tcp_port():
+def get_free_tcp_port() -> int:
     s = socket.socket()
     s.bind(('', 0))
     port = s.getsockname()[1]
@@ -46,14 +46,14 @@ def get_interfaces():
     netifaces = import_netifaces()
     if not netifaces:
         return []
-    return netifaces.interfaces()           #@UndefinedVariable
+    return netifaces.interfaces()           #@UndefinedVariable pylint: disable=no-member
 
-def get_interfaces_addresses():
+def get_interfaces_addresses() -> dict:
     d = {}
     netifaces = import_netifaces()
     if netifaces:
         for iface in get_interfaces():
-            d[iface] = netifaces.ifaddresses(iface)     #@UndefinedVariable
+            d[iface] = netifaces.ifaddresses(iface)     #@UndefinedVariable pylint: disable=no-member
     return d
 
 def get_interface(address):
@@ -70,16 +70,16 @@ def get_interface(address):
                     return iface
     return None
 
-def get_gateways():
+def get_gateways() -> dict:
     netifaces = import_netifaces()
     if not netifaces:
         return {}
     #versions older than 0.10.5 can crash when calling gateways()
     #https://bitbucket.org/al45tair/netifaces/issues/15/gateways-function-crash-segmentation-fault
-    if netifaces.version<'0.10.5':            #@UndefinedVariable
+    if netifaces.version<'0.10.5':            #@UndefinedVariable pylint: disable=no-member
         return {}
     try:
-        d = netifaces.gateways()            #@UndefinedVariable
+        d = netifaces.gateways()            #@UndefinedVariable pylint: disable=no-member
         AF_NAMES = {}
         for k in dir(netifaces):
             if k.startswith("AF_"):
@@ -110,7 +110,7 @@ def do_get_bind_IPs():
     ips = []
     netifaces = import_netifaces()
     assert netifaces
-    ifaces = netifaces.interfaces()            #@UndefinedVariable
+    ifaces = netifaces.interfaces()            #@UndefinedVariable pylint: disable=no-member
     log("ifaces=%s", ifaces)
     for iface in ifaces:
         if_ipmasks = []
@@ -135,7 +135,7 @@ def do_get_bind_ifacemask(iface):
     ipmasks = []
     netifaces = import_netifaces()
     assert netifaces
-    address_types = netifaces.ifaddresses(iface)    #@UndefinedVariable
+    address_types = netifaces.ifaddresses(iface)    #@UndefinedVariable pylint: disable=no-member
     for addresses in address_types.values():
         for address in addresses:
             if 'netmask' in address and 'addr' in address:
@@ -143,22 +143,26 @@ def do_get_bind_ifacemask(iface):
                 mask = address['netmask']
                 if addr!= '::1' and addr != '0.0.0.0' and addr.find("%")<0:
                     try:
-                        fam = socket.AF_INET6 if len(addr.split(":"))>2 else socket.AF_INET
-                        b = socket.inet_pton(fam, addr)
-                        log("socket.inet_pton(AF_INET%s, %s=%s", "6" if fam==socket.AF_INET6 else "", addr, b)
-                        ipmasks.append((addr, mask))
+                        socket.inet_aton(addr)
+                        ipmasks.append((addr,mask))
                     except Exception as e:
-                        log("do_get_bind_ifacemask(%s)", iface, exc_info=True)
-                        log.error("Error converting address '%s' to binary, for interface %s", addr, iface)
+                        log.error("do_get_bind_ifacemask(%s) error on %s", iface, addr, e)
     log("do_get_bind_ifacemask(%s)=%s", iface, ipmasks)
     return ipmasks
 
-def get_iface(ip):
+def get_iface(ip) -> str:
     log("get_iface(%s)", ip)
     if not ip:
         return None
     if ip.find("%")>=0:
-        return ip.split("%", 1)[1]
+        iface = ip.split("%", 1)[1]
+        if if_nametoindex:
+            try:
+                if_nametoindex(iface)
+            except OSError:
+                return None
+            else:
+                return iface
     if ip.find(":")>=0:
         #ipv6?
         return None
@@ -168,6 +172,7 @@ def get_iface(ip):
             v = socket.getaddrinfo(ip, None)
             assert len(v)>0
         except Exception as e:
+            log("socket.getaddrinfo(%s, None)", ip, exc_info=True)
             log.error("Error: cannot revolve '%s'", ip)
             return None
         for i, x in enumerate(v):
@@ -190,30 +195,16 @@ def get_iface(ip):
                 #exact match
                 log("get_iface(%s)=%s", iface, ip)
                 return iface
-            ipv6 = len(test_ip.split(":"))>2
-            if ipv6:
-                test_ip_parts = test_ip.split("/")[0].split(":")
-                mask_parts = mask.split("/")[0].split(":")
-            else:
-                test_ip_parts = test_ip.split(".")
-                mask_parts = mask.split(".")
-                if len(test_ip_parts)!=4 or len(mask_parts)!=4:
-                    log.error("incorrect ip or mask: %s / %s", test_ip, mask)
+            test_ip_parts = test_ip.split(".")
+            mask_parts = mask.split(".")
+            if len(test_ip_parts)!=4 or len(mask_parts)!=4:
+                log.error("incorrect ip or mask: %s/%s", test_ip, mask)
             match = True
             try:
-                for i, ip_part_str in enumerate(test_ip_parts):
-                    if i>=len(mask_parts):
-                        #end of the mask
-                        break
-                    try:
-                        mask_part = int(mask_parts[i] or 0)
-                    except ValueError:
-                        mask_part = int(mask_parts[i], 16)
-                    try:
-                        ip_part = int(ip_part_str or 0)
-                    except ValueError:
-                        ip_part = int(ip_part_str, 16)
-                    test_ip_part = ip_part & mask_part
+                for i in (0,1,2,3):
+                    mask_part = int(mask_parts[i])
+                    ip_part = int(ip_parts[i]) & mask_part
+                    test_ip_part = int(test_ip_parts[i]) & mask_part
                     if ip_part!=test_ip_part:
                         match = False
                         break
@@ -230,7 +221,7 @@ def get_iface(ip):
 if_nametoindex = None
 if_indextoname = None
 
-if WIN32:
+if WIN32:   # pragma: no cover
     def int_if_nametoindex(iface):
         #IPv6 addresses give us the interface as a string:
         #fe80:....%11, so try to convert "11" into 11
@@ -239,7 +230,7 @@ if WIN32:
         except (TypeError, ValueError):
             return None
     if_nametoindex = int_if_nametoindex
-elif PYTHON3:
+else:
     if_nametoindex = socket.if_nametoindex
     def socket_if_indextoname(index):
         if index<0:
@@ -247,40 +238,6 @@ elif PYTHON3:
         return socket.if_indextoname(index)
     if_indextoname = socket_if_indextoname
 
-else:
-    library = "libc.so.6"
-    if OSX:
-        library = "/usr/lib/libc.dylib"
-    elif sys.platform.startswith("sunos"):
-        library = "libsocket.so.1"
-    elif sys.platform.startswith("freebsd"):
-        library = "/lib/libc.so.7"
-    elif sys.platform.startswith("openbsd"):
-        library = "libc.so"
-    try:
-        from ctypes import CDLL, c_char_p, c_uint, create_string_buffer
-        #cdll.LoadLibrary(library)
-        from ctypes.util import find_library
-        #<CDLL 'libc.so.6', handle 7fcac419b000 at 7fcac1ab0c10>
-        _libc = CDLL(find_library(library))
-        log("successfully loaded socket C library from %s", library)
-    except ImportError as e:
-        log.error("library %s not found: %s", library, e)
-    except OSError as e:
-        log.error("error loading %s: %s", library, e)
-    else:
-        _libc.if_indextoname.restype = c_char_p
-        _libc.if_indextoname.argtypes = [c_uint, c_char_p]
-        _libc.if_nametoindex.restype = c_uint
-        _libc.if_nametoindex.argtypes = [c_char_p]
-        def libc_if_nametoindex(interfaceName):
-            return _libc.if_nametoindex(create_string_buffer(interfaceName.encode()))
-        def libc_if_indextoname(index):
-            s = create_string_buffer(b'\000' * 256)
-            return _libc.if_indextoname(c_uint(index), s)
-        if_nametoindex = libc_if_nametoindex
-        if_indextoname = libc_if_indextoname
-
 
 net_sys_config = None
 def get_net_sys_config():
@@ -348,7 +305,7 @@ def get_net_sys_config():
                 addproc("/proc/sys/net/ipv4/%s" % k,     "ipv4", k, int)
     return net_sys_config
 
-def get_net_config():
+def get_net_config() -> dict:
     config = {}
     try:
         from xpra.net.bytestreams import VSOCK_TIMEOUT, SOCKET_TIMEOUT, SOCKET_NODELAY
@@ -358,15 +315,15 @@ def get_net_config():
                 }
         if SOCKET_NODELAY is not None:
             config["socket.nodelay"] = SOCKET_NODELAY
-    except Exception:
+    except Exception:   # pragma: no cover
         log("get_net_config()", exc_info=True)
     return config
 
 
-def get_ssl_info(show_constants=False):
+def get_ssl_info(show_constants=False) -> dict:
     try:
         import ssl
-    except ImportError as e:
+    except ImportError as e:    # pragma: no cover
         log("no ssl: %s", e)
         return {}
     info = {}
@@ -401,7 +358,7 @@ def get_ssl_info(show_constants=False):
     return info
 
 
-def get_network_caps():
+def get_network_caps() -> dict:
     from xpra.net.digest import get_digests
     from xpra.net.crypto import get_crypto_caps
     from xpra.net.compression import get_enabled_compressors, get_compression_caps
@@ -415,13 +372,16 @@ def get_network_caps():
                 "compressors"           : get_enabled_compressors(),
                 "encoders"              : get_enabled_encoders(),
                }
+    from xpra.net.common import FLUSH_HEADER
+    if FLUSH_HEADER:
+        caps["flush"] = True
     caps.update(get_crypto_caps())
     caps.update(get_compression_caps())
     caps.update(get_packet_encoding_caps())
     return caps
 
 
-def get_info():
+def get_info() -> dict:
     i = get_network_caps()
     netifaces = import_netifaces()
     if netifaces:
@@ -443,7 +403,7 @@ def get_info():
     return i
 
 
-def main():
+def main(): # pragma: no cover
     from xpra.os_util import POSIX
     from xpra.util import print_nested_dict, csv
     from xpra.platform import program_context
@@ -464,7 +424,7 @@ def main():
                 print("* %s (index=%s)" % (iface.ljust(20), if_nametoindex(iface)))
             else:
                 print("* %s" % iface)
-            addresses = netifaces.ifaddresses(iface)     #@UndefinedVariable
+            addresses = netifaces.ifaddresses(iface)     #@UndefinedVariable pylint: disable=no-member
             for addr, defs in addresses.items():
                 if addr in (socket.AF_INET, socket.AF_INET6):
                     for d in defs:
@@ -556,5 +516,5 @@ def main():
     return 0
 
 
-if __name__ == "__main__":
+if __name__ == "__main__":  # pragma: no cover
     main()
