@@ -1,28 +1,30 @@
 # This file is part of Xpra.
-# Copyright (C) 2017-2020 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2017-2021 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os.path
 import socket
 from time import sleep
+from ctypes import Structure, c_uint8, sizeof
 
-from xpra.scripts.config import InitException, TRUE_OPTIONS
-from xpra.scripts.main import InitExit
+from xpra.scripts.config import InitException, InitExit, TRUE_OPTIONS
 from xpra.exit_codes import (
     EXIT_SSL_FAILURE, EXIT_SSL_CERTIFICATE_VERIFY_FAILURE,
     EXIT_SERVER_ALREADY_EXISTS, EXIT_SOCKET_CREATION_ERROR,
     )
+from xpra.net.bytestreams import set_socket_timeout, pretty_socket
 from xpra.os_util import (
-    hexstr, bytestostr,
     getuid, get_username_for_uid, get_groups, get_group_id,
     path_permission_info, monotonic_time, umask_context, WIN32, OSX, POSIX,
+    parse_encoded_bin_data,
     )
 from xpra.util import (
     envint, envbool, csv, parse_simple_dict,
-    ellipsizer, repr_ellipsized,
+    ellipsizer,
     DEFAULT_PORT,
     )
+from xpra.make_thread import start_thread
 
 #what timeout value to use on the socket probe attempt:
 WAIT_PROBE_TIMEOUT = envint("XPRA_WAIT_PROBE_TIMEOUT", 6)
@@ -31,6 +33,9 @@ PEEK_TIMEOUT = envint("XPRA_PEEK_TIMEOUT", 1)
 PEEK_TIMEOUT_MS = envint("XPRA_PEEK_TIMEOUT_MS", PEEK_TIMEOUT*1000)
 PEEK_SIZE = envint("XPRA_PEEK_SIZE", 8192)
 
+SOCKET_DIR_MODE = num = int(os.environ.get("XPRA_SOCKET_DIR_MODE", "775"), 8)
+SOCKET_DIR_GROUP = os.environ.get("XPRA_SOCKET_DIR_GROUP", GROUP)
+
 
 network_logger = None
 def get_network_logger():
@@ -42,6 +47,7 @@ def get_network_logger():
 
 
 def create_unix_domain_socket(sockpath, socket_permissions=0o600):
+    assert POSIX
     #convert this to a umask!
     umask = (0o777-socket_permissions) & 0o777
     listener = socket.socket(socket.AF_UNIX)
@@ -79,7 +85,7 @@ def create_unix_domain_socket(sockpath, socket_permissions=0o600):
         delpath = sockpath
         log("cleanup_socket '%s', original inode=%s, new inode=%s", sockpath, inode, cur_inode)
         if cur_inode==inode:
-            log.info("removing socket '%s'", delpath)
+            log.info("removing unix domain socket '%s'", delpath)
             try:
                 os.unlink(delpath)
             except OSError:
@@ -114,9 +120,10 @@ def hosts(host_str):
         return ["0.0.0.0", "::"]
     return [host_str]
 
-def add_listen_socket(socktype, sock, info, new_connection_cb, new_udp_connection_cb=None):
+def add_listen_socket(socktype, sock, info, new_connection_cb, new_udp_connection_cb=None, options=None):
     log = get_network_logger()
-    log("add_listen_socket(%s, %s, %s, %s, %s)", socktype, sock, info, new_connection_cb, new_udp_connection_cb)
+    log("add_listen_socket(%s, %s, %s, %s, %s, %s)",
+        socktype, sock, info, new_connection_cb, new_udp_connection_cb, options)
     try:
         #ugly that we have different ways of starting sockets,
         #TODO: abstract this into the socket class
@@ -125,21 +132,31 @@ def add_listen_socket(socktype, sock, info, new_connection_cb, new_udp_connectio
             sock.new_connection_cb = new_connection_cb
             sock.start()
             return None
+        sources = []
         if socktype=="udp":
             assert new_udp_connection_cb, "UDP sockets cannot be handled here"
             new_udp_connection_cb(sock)
-            return None
-        from gi.repository import GLib
-        sock.listen(5)
-        def io_in_cb(sock, flags):
-            log("io_in_cb(%s, %s)", sock, flags)
-            return new_connection_cb(socktype, sock)
-        source = GLib.io_add_watch(sock, GLib.PRIORITY_DEFAULT, GLib.IO_IN, io_in_cb)
-        sources = [source]
+        else:
+            from gi.repository import GLib
+            sock.listen(5)
+            def io_in_cb(sock, flags):
+                log("io_in_cb(%s, %s)", sock, flags)
+                return new_connection_cb(socktype, sock)
+            source = GLib.io_add_watch(sock, GLib.PRIORITY_DEFAULT, GLib.IO_IN, io_in_cb)
+            sources.append(source)
+        upnp_cleanup = []
+        if socktype in ("udp", "tcp", "ws", "wss", "ssh", "ssl"):
+            upnp = (options or {}).get("upnp", "no")
+            if upnp.lower() in TRUE_OPTIONS:
+                from xpra.net.upnp import upnp_add
+                upnp_cleanup.append(upnp_add(socktype, info, options))
         def cleanup():
             for source in tuple(sources):
                 GLib.source_remove(source)
                 sources.remove(source)
+            for c in upnp_cleanup:
+                if c:
+                    start_thread(c, "pnp-cleanup-%s" % c, daemon=True)
         return cleanup
     except Exception as e:
         log("add_listen_socket(%s, %s)", socktype, sock, exc_info=True)
@@ -175,6 +192,7 @@ def peek_connection(conn, timeout=PEEK_TIMEOUT_MS, size=PEEK_SIZE):
     peek_data = b""
     start = monotonic_time()
     elapsed = 0
+    set_socket_timeout(conn, PEEK_TIMEOUT_MS*1000)
     while elapsed<=timeout:
         try:
             peek_data = conn.peek(size)
@@ -188,19 +206,94 @@ def peek_connection(conn, timeout=PEEK_TIMEOUT_MS, size=PEEK_SIZE):
         sleep(timeout/4000.0)
         elapsed = int(1000*(monotonic_time()-start))
         log("peek: elapsed=%s, timeout=%s", elapsed, timeout)
-    line1 = b""
     log("socket %s peek: got %i bytes", conn, len(peek_data))
-    if peek_data:
-        line1 = peek_data.splitlines()[0]
-        log("socket peek=%s", ellipsizer(peek_data, limit=512))
-        log("socket peek hex=%s", hexstr(peek_data[:128]))
-        log("socket peek line1=%s", ellipsizer(line1))
-    return peek_data, line1
+    return peek_data
+
+
+POSIX_TCP_INFO = (
+        ("state",           c_uint8),
+        )
+def get_sockopt_tcp_info(sock, TCP_INFO, attributes=POSIX_TCP_INFO):
+    def get_tcpinfo_class(fields):
+        class TCPInfo(Structure):
+            _fields_ = tuple(fields)
+            def __repr__(self):
+                return "TCPInfo(%s)" % self.getdict()
+            def getdict(self):
+                return {k[0] : getattr(self, k[0]) for k in self._fields_}
+        return TCPInfo
+    #calculate full structure size with all the fields defined:
+    tcpinfo_class = get_tcpinfo_class(attributes)
+    tcpinfo_size = sizeof(tcpinfo_class)
+    data = sock.getsockopt(socket.SOL_TCP, TCP_INFO, tcpinfo_size)
+    data_size = len(data)
+    #but only define fields in the ctypes.Structure
+    #if they are actually present in the returned data:
+    while tcpinfo_size>data_size:
+        #trim it down:
+        attributes = attributes[:-1]
+        tcpinfo_class = get_tcpinfo_class(attributes)
+        tcpinfo_size = sizeof(tcpinfo_class)
+    log = get_network_logger()
+    if tcpinfo_size==0:
+        log("getsockopt(SOL_TCP, TCP_INFO, %i) no data", tcpinfo_size)
+        return {}
+    #log("total size=%i for fields: %s", size, csv(fdef[0] for fdef in fields))
+    try:
+        tcpinfo = tcpinfo_class.from_buffer_copy(data[:tcpinfo_size])
+    except ValueError as e:
+        log("getsockopt(SOL_TCP, TCP_INFO, %i)", tcpinfo_size, exc_info=True)
+        log("TCPInfo fields=%s", csv(tcpinfo_class._fields_))
+        log.warn("Warning: failed to get TCP_INFO for %s", sock)
+        log.warn(" %s", e)
+        return {}
+    d = tcpinfo.getdict()
+    log("getsockopt(SOL_TCP, TCP_INFO, %i)=%s", tcpinfo_size, d)
+    return d
+
+
+
+def guess_packet_type(data):
+    if not data:
+        return None
+    if data[0]==ord("P"):
+        from xpra.net.header import (
+            unpack_header, HEADER_SIZE,
+            FLAGS_RENCODE, FLAGS_YAML,
+            LZ4_FLAG, LZO_FLAG, BROTLI_FLAG,
+            )
+        header = data.ljust(HEADER_SIZE, b"\0")
+        _, protocol_flags, compression_level, packet_index, data_size = unpack_header(header)
+        #this normally used on the first packet, so the packet index should be 0
+        #and I don't think we can make packets smaller than 8 bytes,
+        #even with packet name aliases and rencode
+        #(and aliases should not be defined for the initial packet anyway)
+        if packet_index==0 and 8<data_size<256*1024*1024:
+            rencode = bool(protocol_flags & FLAGS_RENCODE)
+            yaml = bool(protocol_flags & FLAGS_YAML)
+            lz4 = bool(protocol_flags & LZ4_FLAG)
+            lzo = bool(protocol_flags & LZO_FLAG)
+            brotli = bool(protocol_flags & BROTLI_FLAG)
+            #rencode and yaml are mutually exclusive,
+            #so are the compressors
+            compressors = sum((lz4, lzo, brotli))
+            if not (rencode and yaml) and not compressors>1:
+                #if compression is enabled, the compression level must be set:
+                if not compressors or compression_level>0:
+                    pass #return "xpra"
+    if data[:4]==b"SSH-":
+        return "ssh"
+    if data[0]==0x16:
+        return "ssl"
+    line1 = data.splitlines()[0]
+    if line1.find(b"HTTP/")>0 or line1.split(b" ")[0] in (b"GET", b"POST"):
+        return "http"
+    if line1.lower().startswith(b"<!doctype html") or line1.lower().startswith(b"<html"):
+        return "http"
+    return None
 
 
 def create_sockets(opts, error_cb):
-    log = get_network_logger()
-
     bind_tcp = parse_bind_ip(opts.bind_tcp)
     bind_udp = parse_bind_ip(opts.bind_udp)
     bind_ssl = parse_bind_ip(opts.bind_ssl, 443)
@@ -244,6 +337,7 @@ def create_sockets(opts, error_cb):
             sshlog.error("Error: cannot enable SSH socket upgrades:")
             sshlog.error(" %s", e)
             ssh_upgrades = False
+    log = get_network_logger()
     for socktype, defs in {
         "tcp"   : bind_tcp,
         "ssl"   : bind_ssl,
@@ -336,7 +430,7 @@ def setup_udp_socket(host, iport, socktype="udp"):
     except Exception as e:
         log("create_udp_socket%s", (host, iport), exc_info=True)
         raise InitExit(EXIT_SOCKET_CREATION_ERROR,
-                       "failed to setup %s socket on %s:%s %s" % (socktype, host, iport, e))
+                       "failed to setup %s socket on %s:%s %s" % (socktype, host, iport, e)) from None
     def cleanup_udp_socket():
         log.info("closing %s socket %s:%s", socktype, host, iport)
         try:
@@ -370,8 +464,8 @@ def parse_bind_ip(bind_ip, default_port=DEFAULT_PORT):
                 try:
                     iport = int(port)
                     assert 0<iport<2**16
-                except:
-                    raise InitException("invalid port number: %s" % port)
+                except (TypeError, ValueError):
+                    raise InitException("invalid port number: %s" % port) from None
             options = {}
             if len(parts)==2:
                 options = parse_simple_dict(parts[1])
@@ -410,7 +504,7 @@ def parse_bind_vsock(bind_vsock):
 def setup_sd_listen_socket(stype, sock, addr):
     log = get_network_logger()
     def cleanup_sd_listen_socket():
-        log.info("closing sd listen socket %s", addr)
+        log.info("closing sd listen socket %s", pretty_socket(addr))
         try:
             sock.close()
         except OSError:
@@ -425,7 +519,7 @@ def normalize_local_display_name(local_display_name):
         local_display_name = ":" + local_display_name
     else:
         after_sc = local_display_name[pos+1:]
-    if WIN32:
+    if WIN32 or OSX:
         if after_sc.isalnum():
             return local_display_name
         raise Exception("non alphanumeric character in display name '%s'" % local_display_name)
@@ -438,6 +532,9 @@ def normalize_local_display_name(local_display_name):
 
 def setup_local_sockets(bind, socket_dir, socket_dirs, display_name, clobber,
                         mmap_group="auto", socket_permissions="600", username="", uid=0, gid=0):
+    log = get_network_logger()
+    log("setup_local_sockets%s", (bind, socket_dir, socket_dirs, display_name, clobber,
+                                  mmap_group, socket_permissions, username, uid, gid))
     if not bind:
         return {}
     if not socket_dir and (not socket_dirs or (len(socket_dirs)==1 and not socket_dirs[0])):
@@ -448,13 +545,12 @@ def setup_local_sockets(bind, socket_dir, socket_dirs, display_name, clobber,
                            "at least one socket directory must be set to use unix domain sockets")
     from xpra.platform.dotxpra import DotXpra, norm_makepath
     dotxpra = DotXpra(socket_dir or socket_dirs[0], socket_dirs, username, uid, gid)
-    if display_name is not None:
+    if display_name is not None and not WIN32:
         display_name = normalize_local_display_name(display_name)
-    log = get_network_logger()
     defs = {}
     try:
         sockpaths = {}
-        log("setup_local_sockets: bind=%s", bind)
+        log("setup_local_sockets: bind=%s, dotxpra=%s", bind, dotxpra)
         for b in bind:
             if b in ("none", ""):
                 continue
@@ -491,12 +587,17 @@ def setup_local_sockets(bind, socket_dir, socket_dirs, display_name, clobber,
                 continue
             tmp[sockpath] = options
         sockpaths = tmp
+        log("sockpaths=%s", sockpaths)
         #create listeners:
         if WIN32:
             from xpra.platform.win32.namedpipes.listener import NamedPipeListener
+            from xpra.platform.win32.dotxpra import PIPE_PATH
             for sockpath, options in sockpaths.items():
                 npl = NamedPipeListener(sockpath)
-                log.info("created named pipe '%s'", sockpath)
+                ppath = sockpath
+                if ppath.startswith(PIPE_PATH):
+                    ppath = ppath[len(PIPE_PATH):]
+                log.info("created named pipe '%s'", ppath)
                 defs[("named-pipe", npl, sockpath, npl.stop)] = options
         else:
             def checkstate(sockpath, state):
@@ -523,15 +624,16 @@ def setup_local_sockets(bind, socket_dir, socket_dirs, display_name, clobber,
                 d = os.path.dirname(sockpath)
                 try:
                     kwargs = {}
-                    if getuid()==0 and d=="/var/run/xpra" or d=="/run/xpra":
+                    if d in ("/var/run/xpra", "/run/xpra"):
                         #this is normally done by tmpfiles.d,
                         #but we may need to do it ourselves in some cases:
-                        kwargs = {"mode"  : 0o775}
-                        xpra_gid = get_group_id("xpra")
+                        kwargs["mode"] = SOCKET_DIR_MODE
+                        xpra_gid = get_group_id(SOCKET_DIR_GROUP)
                         if xpra_gid>0:
                             kwargs["gid"] = xpra_gid
                     log("creating sockdir=%s, kwargs=%s" % (d, kwargs))
                     dotxpra.mksockdir(d, **kwargs)
+                    log("%s permission mask: %s", d, oct(os.stat(d).st_mode))
                 except Exception as e:
                     log.warn("Warning: failed to create socket directory '%s'", d)
                     log.warn(" %s", e)
@@ -540,7 +642,6 @@ def setup_local_sockets(bind, socket_dir, socket_dirs, display_name, clobber,
             log("sockets in unknown state: %s", unknown)
             if unknown:
                 #re-probe them using threads so we can do them in parallel:
-                from xpra.make_thread import start_thread
                 threads = []
                 def timeout_probe(sockpath):
                     #we need a loop because "DEAD" sockets may return immediately
@@ -621,7 +722,7 @@ def handle_socket_error(sockpath, sperms, e):
             log.info(" %s does not exist", dirname)
         #only show extra information if the socket permissions
         #would have been accessible by the group:
-        if POSIX and (sperms & 0o40):
+        elif POSIX and (sperms & 0o40):
             uid = getuid()
             username = get_username_for_uid(uid)
             groups = get_groups(username)
@@ -645,19 +746,6 @@ def handle_socket_error(sockpath, sperms, e):
                        "failed to create socket %s" % sockpath)
 
 
-def guess_header_protocol(v):
-    c = int(v[0])
-    s = bytestostr(v)
-    get_network_logger().debug("guess_header_protocol(%r) first char=%#x", repr_ellipsized(s), c)
-    if c==0x16:
-        return "ssl", "SSL packet?"
-    if s[:4]=="SSH-":
-        return "ssh", "SSH packet"
-    if len(s)>=3 and s.split(" ")[0] in ("GET", "POST"):
-        return "HTTP", "HTTP %s request" % s.split(" ")[0]
-    return None, "character %#x, not an xpra client?" % c
-
-
 #warn just once:
 MDNS_WARNING = False
 def mdns_publish(display_name, listen_on, text_dict=None):
@@ -671,7 +759,7 @@ def mdns_publish(display_name, listen_on, text_dict=None):
         from xpra.net import mdns
         assert mdns
         from xpra.net.mdns import XPRA_MDNS_TYPE, RFB_MDNS_TYPE
-        PREFER_ZEROCONF = envbool("XPRA_PREFER_ZEROCONF", False) or WIN32 or OSX
+        PREFER_ZEROCONF = envbool("XPRA_PREFER_ZEROCONF", True)
         if PREFER_ZEROCONF:
             from xpra.net.mdns.zeroconf_publisher import ZeroconfPublishers as MDNSPublishers, get_interface_index
         else:
@@ -762,7 +850,7 @@ def get_ssl_wrap_socket_fn(cert=None, key=None, ca_certs=None, ca_data=None,
                                     options, ciphers,
                                     server_side))
     import ssl
-    ssllog(" verify_mode(%s)=%s", server_side, verify_mode)
+    ssllog(" verify_mode for server_side=%s : %s", server_side, verify_mode)
     #ca-certs:
     if ca_certs=="default":
         ca_certs = None
@@ -780,13 +868,7 @@ def get_ssl_wrap_socket_fn(cert=None, key=None, ca_certs=None, ca_data=None,
         raise InitException("invalid ssl-protocol '%s', must be one of: %s" % (protocol, csv(values)))
     ssllog(" protocol=%#x", proto)
     #ca_data may be hex encoded:
-    if ca_data:
-        import binascii
-        try:
-            ca_data = binascii.unhexlify(ca_data)
-        except (TypeError, binascii.Error):
-            import base64
-            ca_data = base64.b64decode(ca_data)
+    ca_data = parse_encoded_bin_data(ca_data)
     ssllog(" cadata=%s", ellipsizer(ca_data))
 
     kwargs = {
@@ -825,6 +907,7 @@ def get_ssl_wrap_socket_fn(cert=None, key=None, ca_certs=None, ca_data=None,
     ssllog(" cert=%s, key=%s", cert, key)
     if cert:
         SSL_KEY_PASSWORD = os.environ.get("XPRA_SSL_KEY_PASSWORD")
+        ssllog("context.load_cert_chain%s", (cert or None, key or None, SSL_KEY_PASSWORD))
         context.load_cert_chain(certfile=cert or None, keyfile=key or None, password=SSL_KEY_PASSWORD)
     if ssl_cert_reqs!=ssl.CERT_NONE:
         if server_side:
@@ -832,7 +915,7 @@ def get_ssl_wrap_socket_fn(cert=None, key=None, ca_certs=None, ca_data=None,
         else:
             purpose = ssl.Purpose.SERVER_AUTH   #@UndefinedVariable
             context.check_hostname = check_hostname
-            ssllog(" check_hostname=%s", check_hostname)
+            ssllog(" check_hostname=%s, server_hostname=%s", check_hostname, server_hostname)
             if context.check_hostname:
                 if not server_hostname:
                     raise InitException("ssl error: check-hostname is set but server-hostname is not")
@@ -841,8 +924,8 @@ def get_ssl_wrap_socket_fn(cert=None, key=None, ca_certs=None, ca_data=None,
         context.load_default_certs(purpose)
 
         if not ca_certs or ca_certs.lower()=="default":
-            ssllog(" loading default verify paths")
-            context.set_default_verify_paths()
+            ssllog(" using default certs")
+            #load_default_certs already calls set_default_verify_paths()
         elif not os.path.exists(ca_certs):
             raise InitException("invalid ssl-ca-certs file or directory: %s" % ca_certs)
         elif os.path.isdir(ca_certs):
@@ -864,7 +947,7 @@ def get_ssl_wrap_socket_fn(cert=None, key=None, ca_certs=None, ca_data=None,
                 f.file.flush()
                 context.load_verify_locations(cafile=f.name)
     elif check_hostname and not server_side:
-        raise InitException("cannot check hostname with verify mode %s" % verify_mode)
+        ssllog("cannot check hostname client side with verify mode %s", verify_mode)
     wrap_socket = context.wrap_socket
     def do_wrap_socket(tcp_socket):
         assert tcp_socket
@@ -906,4 +989,3 @@ def get_ssl_wrap_socket_fn(cert=None, key=None, ca_certs=None, ca_data=None,
                 raise InitExit(status, "SSL handshake failed: %s" % msg)
         return ssl_sock
     return do_wrap_socket
-
