@@ -87,7 +87,10 @@ def do_init():
     mh = getOSXMenuHelper(None)
     log("do_init() menu helper=%s", mh)
     osxapp.set_dock_menu(mh.build_dock_menu())
-    osxapp.set_menu_bar(mh.rebuild())
+    import warnings
+    with warnings.catch_warnings():
+        warnings.filterwarnings("ignore", message=".*invalid cast from 'GtkMenuBar'")
+        osxapp.set_menu_bar(mh.rebuild())
 
 
 def do_ready():
@@ -204,7 +207,7 @@ def get_double_click_time():
         #(but still call it "Time" you see)
         MS_PER_TICK = 1000.0/60
         return int(GetDblTime() * MS_PER_TICK)
-    except:
+    except Exception:
         return -1
 
 
@@ -235,7 +238,7 @@ def get_window_frame_size(x, y, w, h):
                 "offset"    : (dx+dw//2, dy+dh),
                 "frame"     : (dx+dw//2, dx+dw//2, dy+dh, dy),
                 }
-    except:
+    except Exception:
         log("failed to query frame size using Quartz, using hardcoded value", exc_info=True)
         return {            #left, right, top, bottom:
                 "offset"    : (0, 22),
@@ -257,11 +260,7 @@ def get_workareas():
         frame = screen.frame()
         visibleFrame = screen.visibleFrame()
         log(" frame=%s, visibleFrame=%s", frame, visibleFrame)
-        try:
-            #10.7 onwards:
-            log(" backingScaleFactor=%s", screen.backingScaleFactor())
-        except:
-            pass
+        log(" backingScaleFactor=%s", screen.backingScaleFactor())
         x = int(visibleFrame.origin.x - frame.origin.x)
         y = int((frame.size.height - visibleFrame.size.height) - (visibleFrame.origin.y - frame.origin.y))
         w = int(visibleFrame.size.width)
@@ -695,6 +694,7 @@ class ClientExtras:
         self.event_loop_started = False
         self.check_display_timer = 0
         self.display_is_asleep = False
+        self.shared_app = None
         if opts and client:
             log("setting swap_keys=%s using %s", swap_keys, client.keyboard_helper)
             if client.keyboard_helper and client.keyboard_helper.keyboard:
@@ -723,15 +723,11 @@ class ClientExtras:
     def ready(self):
         try:
             self.setup_event_listener()
-        except:
+        except Exception:
             log.error("Error setting up OSX event listener", exc_info=True)
 
     def setup_event_listener(self):
         log("setup_event_listener()")
-        if NSObject is object:
-            log.warn("NSObject is missing, not setting up OSX event listener")
-            return
-        self.shared_app = None
         self.delegate = None
         self.shared_app = NSApplication.sharedApplication()
 
