@@ -1,9 +1,14 @@
 # This file is part of Xpra.
-# Copyright (C) 2012-2019 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2012-2020 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-from AppKit import NSStringPboardType, NSPasteboard     #@UnresolvedImport
+from io import BytesIO
+from AppKit import (
+    NSStringPboardType, NSTIFFPboardType, NSPasteboardTypePNG, NSPasteboardTypeURL,  #@UnresolvedImport
+    NSPasteboard,       #@UnresolvedImport
+    )
+from CoreFoundation import NSData, CFDataGetBytes, CFDataGetLength  #@UnresolvedImport
 from gi.repository import GLib
 
 from xpra.clipboard.clipboard_timeout_helper import ClipboardTimeoutHelper
@@ -70,11 +75,7 @@ class OSXClipboardProxy(ClipboardProxyCore):
         self.update_change_count()
         #setup clipboard counter watcher:
         w = get_UI_watcher(GLib.timeout_add, GLib.source_remove)
-        if w is None:
-            log.warn("Warning: no UI watcher instance available")
-            log.warn(" cannot detect clipboard change events")
-        else:
-            w.add_alive_callback(self.timer_clipboard_check)
+        w.add_alive_callback(self.timer_clipboard_check)
 
     def cleanup(self):
         ClipboardProxyCore.cleanup(self)
@@ -103,10 +104,10 @@ class OSXClipboardProxy(ClipboardProxyCore):
     def do_emit_token(self):
         packet_data = []
         if self._want_targets:
-            targets = filter_targets(self.pasteboard.types())
+            targets = self.get_targets()
             log("do_emit_token() targets=%s", targets)
             packet_data.append(targets)
-            if self._greedy_client:
+            if self._greedy_client and "TEXT" in targets:
                 text = self.get_clipboard_text()
                 if text:
                     packet_data += ["STRING", "bytes", 8, text]
@@ -118,19 +119,62 @@ class OSXClipboardProxy(ClipboardProxyCore):
         log("get_clipboard_text() NSStringPboardType='%s' (%s)", text, type(text))
         return str(text)
 
+    def get_targets(self):
+        types = self.pasteboard.types()
+        targets = []
+        if any(t in (NSStringPboardType, NSPasteboardTypeURL, "public.utf8-plain-text", "public.html", "TEXT") for t in types):
+            targets += ["TEXT", "STRING", "text/plain", "text/plain;charset=utf-8", "UTF8_STRING"]
+        if any(t in (NSTIFFPboardType, NSPasteboardTypePNG) for t in types):
+            targets += ["image/png", "image/jpeg", "image/tiff"]
+        log("get_targets() targets(%s)=%s", types, targets)
+        return targets
+
     def get_contents(self, target, got_contents):
         log("get_contents%s", (target, got_contents))
         if target=="TARGETS":
-            #we only support text at the moment:
-            got_contents("ATOM", 32, ["TEXT", "STRING", "text/plain", "text/plain;charset=utf-8", "UTF8_STRING"])
+            got_contents("ATOM", 32, self.get_targets())
             return
-        if target not in ("TEXT", "STRING", "text/plain", "text/plain;charset=utf-8", "UTF8_STRING"):
-            #we don't know how to handle this target,
-            #return an empty response:
-            got_contents(target, 8, b"")
+        if target in ("image/png", "image/jpeg", "image/tiff"):
+            try:
+                data = self.get_image_contents(target)
+            except Exception:
+                log.error("Error: failed to copy image from clipboard", exc_info=True)
+            if data:
+                got_contents(target, 8, data)
+                return
+        if target in ("TEXT", "STRING", "text/plain", "text/plain;charset=utf-8", "UTF8_STRING"):
+            text = self.get_clipboard_text()
+            got_contents(target, 8, text)
             return
-        text = self.get_clipboard_text()
-        got_contents(target, 8, text)
+        #we don't know how to handle this target,
+        #return an empty response:
+        got_contents(target, 8, b"")
+
+    def get_image_contents(self, target):
+        types = filter_targets(self.pasteboard.types())
+        if target=="image/png" and NSPasteboardTypePNG in types:
+            src_dtype = "image/png"
+            img_data = self.pasteboard.dataForType_(NSPasteboardTypePNG)
+        elif target=="image/tiff" and NSTIFFPboardType in types:
+            src_dtype = "image/tiff"
+            img_data = self.pasteboard.dataForType_(NSTIFFPboardType)
+        elif NSPasteboardTypePNG in types:
+            src_dtype = "image/png"
+            img_data = self.pasteboard.dataForType_(NSPasteboardTypePNG)
+        elif NSTIFFPboardType in types:
+            src_dtype = "image/tiff"
+            img_data = self.pasteboard.dataForType_(NSTIFFPboardType)
+        else:
+            log("image target '%s' not found in %s", target, types)
+            return None
+        if not img_data:
+            return None
+        l = CFDataGetLength(img_data)
+        img_data = CFDataGetBytes(img_data, (0, l), None)
+        img_data = self.filter_data(dtype=src_dtype, dformat=8, data=img_data, trusted=False, output_dtype=target)
+        log("get_image_contents(%s)=%i %s", target, len(img_data or ()), type(img_data))
+        return img_data
+
 
     def got_token(self, targets, target_data=None, claim=True, _synchronous_client=False):
         # the remote end now owns the clipboard
@@ -165,11 +209,47 @@ class OSXClipboardProxy(ClipboardProxyCore):
         #if this is the special target 'TARGETS', cache the result:
         if target=="TARGETS" and dtype=="ATOM" and dformat==32:
             self.targets = _filter_targets(data)
-            #TODO: tell system what targets we have
             log("got_contents: tell OS we have %s", csv(self.targets))
+            image_types = tuple(t for t in ("image/png", "image/jpeg", "image/tiff") if t in self.targets)
+            log("image_types=%s, dtype=%s (is text=%s)",
+                     image_types, dtype, dtype in TEXT_TARGETS)
+            if image_types and dtype not in TEXT_TARGETS:
+                #request image:
+                self.send_clipboard_request_handler(self, self._selection, image_types[0])
+            return
         if dformat==8 and dtype in TEXT_TARGETS:
             log("we got a byte string: %s", data)
             self.set_clipboard_text(data)
+        if dformat==8 and dtype in ("image/png", "image/jpeg", "image/tiff"):
+            log("we got a %s image", dtype)
+            self.set_image_data(dtype, data)
+
+    def set_image_data(self, dtype, data):
+        img_type = dtype.split("/")[1]      #ie: "png"
+        from xpra.codecs.pillow.decoder import open_only
+        img = open_only(data, (img_type, ))
+        for img_type, macos_types in {
+            "png"   : [NSPasteboardTypePNG, "image/png"],
+            "tiff"  : [NSTIFFPboardType, "image/tiff"],
+            "jpeg"  : ["public.jpeg", "image/jpeg"],
+            }.items():
+            try:
+                save_img = img
+                if img_type=="jpeg" and img.mode=="RGBA":
+                    save_img = img.convert("RGB")
+                buf = BytesIO()
+                save_img.save(buf, img_type)
+                data = buf.getvalue()
+                buf.close()
+                self.pasteboard.clearContents()
+                nsdata = NSData.dataWithData_(data)
+                for t in macos_types:
+                    r = self.pasteboard.setData_forType_(nsdata, t)
+                    log("set '%s' data type: %s", t, r)
+            except Exception as e:
+                log("set_image_data(%s, ..)", dtype, exc_info=True)
+                log.error("Error: failed to copy %s image to clipboard", img_type)
+                log.error(" %s", e)
 
     def set_clipboard_text(self, text):
         self.pasteboard.clearContents()
