@@ -61,6 +61,9 @@ class Keyboard(KeyboardBase):
         KEY_TRANSLATIONS[("dead_tilde", 65107,  50)]    = "asciitilde"
         KEY_TRANSLATIONS[("dead_grave", 65104,  55)]    = "grave"
 
+    def __repr__(self):
+        return "win32.Keyboard"
+
     def set_modifier_mappings(self, mappings):
         super().set_modifier_mappings(mappings)
         self.num_lock_modifier = self.modifier_keys.get("Num_Lock")
@@ -88,7 +91,7 @@ class Keyboard(KeyboardBase):
                 elif not numlock and self.num_lock_modifier in names:
                     names.remove(self.num_lock_modifier)
                 log("mask_to_names(%s) GetKeyState(VK_NUMLOCK)=%s, names=%s", mask, numlock, names)
-            except:
+            except Exception:
                 pass
         else:
             log("mask_to_names(%s)=%s", mask, names)
