@@ -126,6 +126,9 @@ def display_name_check(display_name):
     n = display_name[1:].split(".")[0]    #ie: ":0.0" -> "0"
     try:
         dno = int(n)
+    except (ValueError, TypeError):
+        raise InitException("invalid display number %r" % n) from None
+    else:
         if 0<=dno<10:
             warn("WARNING: low display number: %s" % dno)
             warn(" You are attempting to run the xpra server")
@@ -133,8 +136,6 @@ def display_name_check(display_name):
             warn(" This is generally not what you want.")
             warn(" You should probably use a higher display number")
             warn(" just to avoid any confusion and this warning message.")
-    except IOError:
-        pass
 
 
 def print_DE_warnings():
@@ -267,16 +268,17 @@ def show_encoding_help(opts):
     set_default_level(logging.WARN)
     logging.root.setLevel(logging.WARN)
     for x in get_all_loggers():
-        x.logger.setLevel(logging.WARN)
+        if x.logger.getEffectiveLevel()==logging.INFO:
+            x.logger.setLevel(logging.WARN)
     from xpra.server.server_base import ServerBase
     sb = ServerBase()
     sb.init(opts)
-    from xpra.codecs.codec_constants import PREFERED_ENCODING_ORDER, HELP_ORDER
+    from xpra.codecs.codec_constants import PREFERRED_ENCODING_ORDER, HELP_ORDER
     if "help" in opts.encodings:
-        sb.allowed_encodings = PREFERED_ENCODING_ORDER
-    from xpra.codecs.video_helper import getVideoHelper
-    getVideoHelper().init()
-    sb.init_encodings()
+        sb.allowed_encodings = PREFERRED_ENCODING_ORDER
+    from xpra.server.mixins.encoding_server import EncodingServer
+    assert isinstance(sb, EncodingServer)
+    EncodingServer.threaded_setup(sb)
     from xpra.codecs.loader import encoding_help
     for e in (x for x in HELP_ORDER if x in sb.encodings):
         print(" * %s" % encoding_help(e))
@@ -313,6 +315,7 @@ def set_server_features(opts):
     server_features.encoding        = impcheck("codecs")
     server_features.logging         = b(opts.remote_logging)
     #server_features.network_state   = ??
+    server_features.shell           = envbool("XPRA_SHELL", True)
     server_features.display         = opts.windows
     server_features.windows         = opts.windows and impcheck("codecs")
     server_features.rfb             = b(opts.rfb_upgrade) and impcheck("server.rfb")
@@ -335,21 +338,12 @@ def make_proxy_server():
     return ProxyServer()
 
 
-def run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=None):
-    #add finally hook to ensure we will run the cleanups
-    #even if we exit because of an exception:
-    try:
-        return do_run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display)
-    finally:
-        run_cleanups()
-        import gc
-        gc.collect()
-
-
-def verify_display(xvfb=None, display_name=None, shadowing=False, log_errors=True):
+def verify_display(xvfb=None, display_name=None, shadowing=False, log_errors=True, timeout=None):
     #check that we can access the X11 display:
-    from xpra.x11.vfb_util import verify_display_ready
-    if not verify_display_ready(xvfb, display_name, shadowing, log_errors):
+    from xpra.x11.vfb_util import verify_display_ready, VFB_WAIT
+    if timeout is None:
+        timeout = VFB_WAIT
+    if not verify_display_ready(xvfb, display_name, shadowing, log_errors, timeout):
         return 1
     from xpra.log import Logger
     log = Logger("screen", "x11")
@@ -362,13 +356,19 @@ def verify_display(xvfb=None, display_name=None, shadowing=False, log_errors=Tru
     log("GDK can access the display")
     return 0
 
-def do_run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=None):
+def do_run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=None, progress_cb=None):
     assert mode in (
         "start", "start-desktop",
         "upgrade", "upgrade-desktop",
         "shadow", "proxy",
         )
 
+    def _progress(i, msg):
+        if progress_cb:
+            progress_cb(i, msg)
+    progress = _progress
+
+    progress(10, "initializing environment")
     try:
         cwd = os.getcwd()
     except OSError:
@@ -396,13 +396,21 @@ def do_run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=N
     if not proxying and POSIX and not OSX:
         #we don't support wayland servers,
         #so make sure GDK will use the X11 backend:
+        from xpra.os_util import saved_env
+        saved_env["GDK_BACKEND"] = "x11"
         os.environ["GDK_BACKEND"] = "x11"
 
+    has_child_arg = (
+            opts.start_child or
+            opts.start_child_on_connect or
+            opts.start_child_after_connect or
+            opts.start_child_on_last_client_exit
+            )
     if proxying or upgrading or upgrading_desktop:
         #when proxying or upgrading, don't exec any plain start commands:
         opts.start = opts.start_child = []
     elif opts.exit_with_children:
-        assert opts.start_child, "exit-with-children was specified but start-child is missing!"
+        assert has_child_arg, "exit-with-children was specified but start-child* is missing!"
     elif opts.start_child:
         warn("Warning: the 'start-child' option is used,")
         warn(" but 'exit-with-children' is not enabled,")
@@ -425,7 +433,7 @@ def do_run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=N
     if shadowing and not extra_args:
         if WIN32 or OSX:
             #just a virtual name for the only display available:
-            display_name = ":0"
+            display_name = "Main"
         else:
             from xpra.scripts.main import guess_X11_display
             dotxpra = DotXpra(opts.socket_dir, opts.socket_dirs)
@@ -437,7 +445,7 @@ def do_run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=N
             error_cb("too many extra arguments (%i): only expected a display number" % len(extra_args))
         if len(extra_args) == 1:
             display_name = extra_args[0]
-            if not shadowing and not proxying and not upgrading and not use_display:
+            if not shadowing and not upgrading and not use_display:
                 display_name_check(display_name)
         else:
             if proxying:
@@ -463,7 +471,7 @@ def do_run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=N
                 display_name = 'S' + str(os.getpid())
 
     if not (shadowing or proxying or upgrading or upgrading_desktop) and \
-    opts.exit_with_children and not opts.start_child:
+    opts.exit_with_children and not has_child_arg:
         error_cb("--exit-with-children specified without any children to spawn; exiting immediately")
 
     atexit.register(run_cleanups)
@@ -621,15 +629,18 @@ def do_run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=N
         os.environ["XPRA_SERVER_LOG"] = log_filename0
     else:
         #server log does not exist:
-        try:
-            del os.environ["XPRA_SERVER_LOG"]
-        except KeyError:
-            pass
+        os.environ.pop("XPRA_SERVER_LOG", None)
 
     #warn early about this:
     if (starting or starting_desktop) and desktop_display and opts.notifications and not opts.dbus_launch:
         print_DE_warnings()
 
+    if start_vfb and opts.xvfb.find("Xephyr")>=0 and opts.sync_xvfb<=0:
+        warn("Warning: using Xephyr as vfb")
+        warn(" you should also enable the sync-xvfb option")
+        warn(" to keep the Xephyr window updated")
+
+    progress(10, "creating sockets")
     from xpra.net.socket_util import get_network_logger, setup_local_sockets, create_sockets
     sockets = create_sockets(opts, error_cb)
 
@@ -647,11 +658,8 @@ def do_run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=N
         os.environ["DISPLAY"] = display_name
         if POSIX:
             os.environ["CKCON_X11_DISPLAY"] = display_name
-    else:
-        try:
-            del os.environ["DISPLAY"]
-        except KeyError:
-            pass
+    elif not start_vfb or opts.xvfb.find("Xephyr")<0:
+        os.environ.pop("DISPLAY", None)
     os.environ.update(protected_env)
     from xpra.log import Logger
     log = Logger("server")
@@ -671,17 +679,24 @@ def do_run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=N
         if not display_name:
             use_display = False
         else:
-            start_vfb = verify_display(None, display_name, log_errors=False)!=0
+            progress(20, "connecting to the display")
+            start_vfb = verify_display(None, display_name, log_errors=False, timeout=1)!=0
     if start_vfb:
+        progress(20, "starting a virtual display")
         assert not proxying and xauth_data
         pixel_depth = validate_pixel_depth(opts.pixel_depth, starting_desktop)
-        from xpra.x11.vfb_util import start_Xvfb, check_xvfb_process
+        from xpra.x11.vfb_util import start_Xvfb, check_xvfb_process, parse_resolution
         from xpra.server.server_util import has_uinput
         uinput_uuid = None
         if has_uinput() and opts.input_devices.lower() in ("uinput", "auto") and not shadowing:
             from xpra.os_util import get_rand_chars
             uinput_uuid = get_rand_chars(UINPUT_UUID_LEN)
-        xvfb, display_name, cleanups = start_Xvfb(opts.xvfb, pixel_depth, display_name, cwd,
+        vfb_geom = ""
+        try:
+            vfb_geom = parse_resolution(opts.resize_display)
+        except Exception:
+            pass
+        xvfb, display_name, cleanups = start_Xvfb(opts.xvfb, vfb_geom, pixel_depth, display_name, cwd,
                                                   uid, gid, username, xauth_data, uinput_uuid)
         for f in cleanups:
             add_cleanup(f)
@@ -694,7 +709,7 @@ def do_run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=N
             pam.set_items({"XDISPLAY" : display_name})
 
         def check_xvfb(timeout=0):
-            return check_xvfb_process(xvfb, timeout=timeout)
+            return check_xvfb_process(xvfb, timeout=timeout, command=opts.xvfb)
     else:
         if POSIX and clobber:
             #if we're meant to be using a private XAUTHORITY file,
@@ -827,6 +842,7 @@ def do_run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=N
         log("gui_init()")
         gui_init()
 
+    progress(50, "creating local sockets")
     #setup unix domain socket:
     netlog = get_network_logger()
     local_sockets = setup_local_sockets(opts.bind,
@@ -867,6 +883,7 @@ def do_run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=N
         if uinput_uuid:
             save_uinput_id(uinput_uuid)
 
+    progress(60, "initializing server")
     if shadowing:
         app = make_shadow_server()
     elif proxying:
@@ -883,14 +900,16 @@ def do_run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=N
         app.exec_cwd = opts.chdir or cwd
         app.display_name = display_name
         app.init(opts)
+        progress(70, "initializing sockets")
         app.init_sockets(sockets)
         app.init_dbus(dbus_pid, dbus_env)
-        if not shadowing and (xvfb_pid or clobber):
+        if not shadowing and not proxying:
             app.init_display_pid(xvfb_pid)
         app.original_desktop_display = desktop_display
         del opts
         if not app.server_ready():
             return 1
+        progress(80, "finalizing")
         app.server_init()
         app.setup()
         app.init_when_ready(_when_ready)
@@ -912,6 +931,7 @@ def do_run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=N
         return 1
 
     try:
+        progress(100, "running")
         log("running %s", app.run)
         r = app.run()
         log("%s()=%s", app.run, r)
