@@ -1,17 +1,17 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2010-2018 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2020 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 #pylint: disable-msg=E1101
 
 import os.path
 
-from xpra.util import parse_scaling_value, from0to100
-from xpra.server.control_command import ArgsControlCommand, ControlError
-from xpra.os_util import load_binary_file
-from xpra.util import csv
+from xpra.util import parse_scaling_value, csv, from0to100
+from xpra.os_util import load_binary_file, bytestostr, strtobytes
+from xpra.simple_stats import std_unit
 from xpra.scripts.config import parse_bool, FALSE_OPTIONS, TRUE_OPTIONS
+from xpra.server.control_command import ArgsControlCommand, ControlError
 from xpra.server.mixins.stub_server_mixin import StubServerMixin
 from xpra.log import Logger
 
@@ -58,6 +58,8 @@ class ServerBaseControlCommands(StubServerMixin):
 
         for cmd in (
             ArgsControlCommand("focus",                 "give focus to the window id",      validation=[int]),
+            ArgsControlCommand("map",                   "maps the window id",               validation=[int]),
+            ArgsControlCommand("unmap",                 "unmaps the window id",             validation=[int]),
             #window source:
             ArgsControlCommand("suspend",               "suspend screen updates",           max_args=0),
             ArgsControlCommand("resume",                "resume screen updates",            max_args=0),
@@ -89,6 +91,9 @@ class ServerBaseControlCommands(StubServerMixin):
             ArgsControlCommand("sound-output",          "control sound forwarding",         min_args=1, max_args=2),
             #windows:
             ArgsControlCommand("workspace",             "move a window to a different workspace", min_args=2, max_args=2, validation=[int, int]),
+            ArgsControlCommand("move",                  "move a window",                    min_args=3, max_args=3, validation=[int, int, int]),
+            ArgsControlCommand("resize",                "resize a window",                  min_args=3, max_args=3, validation=[int, int, int]),
+            ArgsControlCommand("moveresize",            "move and resize a window",         min_args=5, max_args=5, validation=[int, int, int, int, int]),
             ArgsControlCommand("scaling-control",       "set the scaling-control aggressiveness (from 0 to 100)", min_args=1, validation=[from0to100]),
             ArgsControlCommand("scaling",               "set a specific scaling value",     min_args=1, validation=[parse_scaling_value]),
             ArgsControlCommand("auto-refresh",          "set a specific auto-refresh value", min_args=1, validation=[float]),
@@ -122,6 +127,44 @@ class ServerBaseControlCommands(StubServerMixin):
         self._focus(None, wid, None)
         return "gave focus to window %s" % wid
 
+    def control_command_map(self, wid):
+        if self.readonly:
+            return
+        assert type(wid)==int, "argument should have been an int, but found %s" % type(wid)
+        window = self._id_to_window.get(wid)
+        assert window, "window %i not found" % wid
+        if window.is_tray():
+            return "cannot map tray window %s" % wid
+        if window.is_OR():
+            return "cannot map override redirect window %s" % wid
+        dm = getattr(self, "_desktop_manager", None)
+        assert dm, "%r does not have a desktop manager" % self
+        dm.show_window(window)
+        #window.set_owner(dm)
+        #iconic = window.get_property("iconic")
+        #if iconic:
+        #    window.set_property("iconic", False)
+        #w, h = window.get_geometry()[2:4]
+        #self.refresh_window_area(window, 0, 0, w, h)
+        self.repaint_root_overlay()
+        return "mapped window %s" % wid
+
+    def control_command_unmap(self, wid):
+        if self.readonly:
+            return
+        assert type(wid)==int, "argument should have been an int, but found %s" % type(wid)
+        window = self._id_to_window.get(wid)
+        assert window, "window %i not found" % wid
+        if window.is_tray():
+            return "cannot map tray window %s" % wid
+        if window.is_OR():
+            return "cannot map override redirect window %s" % wid
+        dm = getattr(self, "_desktop_manager", None)
+        assert dm, "%r does not have a desktop manager" % self
+        dm.hide_window(window)
+        self.repaint_root_overlay()
+        return "unmapped window %s" % wid
+
     def control_command_suspend(self):
         for csource in tuple(self._server_sources.values()):
             csource.suspend(True, self._id_to_window)
@@ -241,6 +284,13 @@ class ServerBaseControlCommands(StubServerMixin):
         return "url sent to %i clients" % clients
 
     def control_command_send_file(self, filename, openit="open", client_uuids="*", maxbitrate=0):
+        #we always get the values as strings from the command interface,
+        #but those may actually be utf8 encoded binary strings,
+        #so we may have to do an ugly roundtrip:
+        try:
+            filename = strtobytes(filename).decode("utf8")
+        except:
+            filename = bytestostr(filename)
         openit = str(openit).lower() in ("open", "true", "1")
         return self.do_control_file_command("send file", client_uuids, filename, "file_transfer", (False, openit))
 
@@ -260,10 +310,9 @@ class ServerBaseControlCommands(StubServerMixin):
         if not sources:
             raise ControlError("no clients found matching: %s" % client_uuids)
         def checksize(file_size):
-            file_size_MB = file_size//1024//1024
-            if file_size_MB>self.file_transfer.file_size_limit:
-                raise ControlError("file '%s' is too large: %iMB (limit is %iMB)" % (
-                    filename, file_size_MB, self.file_transfer.file_size_limit))
+            if file_size>self.file_transfer.file_size_limit:
+                raise ControlError("file '%s' is too large: %sB (limit is %sB)" % (
+                    filename, std_unit(file_size), std_unit(self.file_transfer.file_size_limit)))
 
         #find the file and load it:
         actual_filename = os.path.abspath(os.path.expanduser(filename))
@@ -281,7 +330,6 @@ class ServerBaseControlCommands(StubServerMixin):
             raise ControlError("failed to load '%s'" % actual_filename)
         #verify size:
         file_size = len(data)
-        file_size_MB = file_size//1024//1024
         checksize(file_size)
         #send it to each client:
         for ss in sources:
@@ -289,9 +337,10 @@ class ServerBaseControlCommands(StubServerMixin):
             if not getattr(ss, source_flag_name):
                 log.warn("Warning: cannot %s '%s'", command_type, filename)
                 log.warn(" client %s does not support this feature", ss)
-            elif file_size_MB>ss.file_size_limit:
+            elif file_size>ss.file_size_limit:
                 log.warn("Warning: cannot %s '%s'", command_type, filename)
-                log.warn(" client %s file size limit is %iMB (file is %iMB)", ss, ss.file_size_limit, file_size_MB)
+                log.warn(" client %s file size limit is %sB (file is %sB)",
+                         ss, std_unit(ss.file_size_limit), std_unit(file_size))
             else:
                 ss.send_file(filename, "", data, file_size, *send_file_args)
         return "%s of '%s' to %s initiated" % (command_type, filename, client_uuids)
@@ -371,6 +420,7 @@ class ServerBaseControlCommands(StubServerMixin):
         log.info("changed session name: %s", self.session_name)
         #self.all_send_client_command("name", name)    not supported by any clients, don't bother!
         self.setting_changed("session_name", name)
+        self.mdns_update()
         return "session name set to %s" % name
 
     def _control_windowsources_from_args(self, *args):
@@ -585,6 +635,44 @@ class ServerBaseControlCommands(StubServerMixin):
         return "window %s moved to workspace %s" % (wid, workspace)
 
 
+    def control_command_move(self, wid, x, y):
+        window = self._id_to_window.get(wid)
+        if not window:
+            raise ControlError("window %s does not exist" % wid)
+        ww, wh = window.get_dimensions()
+        count = 0
+        for source in tuple(self._server_sources.values()):
+            move_resize_window = getattr(source, "move_resize_window", None)
+            if move_resize_window:
+                move_resize_window(wid, window, x, y, ww, wh)
+                count += 1
+        return "window %s moved to %i,%i for %i clients" % (wid, x, y, count)
+
+    def control_command_resize(self, wid, w, h):
+        window = self._id_to_window.get(wid)
+        if not window:
+            raise ControlError("window %s does not exist" % wid)
+        count = 0
+        for source in tuple(self._server_sources.values()):
+            resize_window = getattr(source, "resize_window", None)
+            if resize_window:
+                resize_window(wid, window, w, h)
+                count += 1
+        return "window %s resized to %ix%i for %i clients" % (wid, w, h, count)
+
+    def control_command_moveresize(self, wid, x, y, w, h):
+        window = self._id_to_window.get(wid)
+        if not window:
+            raise ControlError("window %s does not exist" % wid)
+        count = 0
+        for source in tuple(self._server_sources.values()):
+            move_resize_window = getattr(source, "move_resize_window", None)
+            if move_resize_window:
+                move_resize_window(wid, window, x, y, w, h)
+                count += 1
+        return "window %s moved to %i,%i and resized to %ix%i for %i clients" % (wid, x, y, w, h, count)
+
+
     def _process_command_request(self, _proto, packet):
         """ client sent a command request through its normal channel """
         assert len(packet)>=2, "invalid command request packet (too small!)"
