@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2010-2018 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2020 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -8,18 +8,22 @@ import os
 
 from xpra.net.compression import Compressed
 from xpra.server.source.stub_source_mixin import StubSourceMixin
-from xpra.os_util import get_machine_id, get_user_uuid, bytestostr, POSIX, PYTHON3
-from xpra.util import csv, envbool, envint, flatten_dict, typedict, XPRA_AUDIO_NOTIFICATION_ID
+from xpra.os_util import get_machine_id, get_user_uuid, bytestostr, POSIX
+from xpra.util import csv, envbool, flatten_dict, typedict, XPRA_AUDIO_NOTIFICATION_ID
 from xpra.log import Logger
 
 log = Logger("sound")
 
 NEW_STREAM_SOUND = envbool("XPRA_NEW_STREAM_SOUND", True)
-NEW_STREAM_SOUND_STOP = envint("XPRA_NEW_STREAM_SOUND_STOP", 20)
 
 
 class AudioMixin(StubSourceMixin):
 
+    @classmethod
+    def is_needed(cls, caps : typedict) -> bool:
+        return caps.boolget("sound.send") or caps.boolget("sound.receive")
+
+
     def __init__(self):
         self.sound_properties = {}
         self.sound_source_plugin = ""
@@ -50,45 +54,22 @@ class AudioMixin(StubSourceMixin):
         self.sound_send = False
         self.sound_bundle_metadata = False
         self.sound_fade_timer = None
-        self.new_stream_timers = {}
 
     def cleanup(self):
         log("%s.cleanup()", self)
         self.cancel_sound_fade_timer()
         self.stop_sending_sound()
         self.stop_receiving_sound()
-        self.stop_new_stream_notifications()
         self.init_state()
 
 
-    def stop_new_stream_notifications(self):
-        timers = self.new_stream_timers.copy()
-        self.new_stream_timers = {}
-        for proc, timer in timers.items():
-            timer = self.new_stream_timers.pop(proc, None)
-            if timer:
-                self.source_remove(timer)
-            self.stop_new_stream_notification(proc)
-
-    def stop_new_stream_notification(self, proc):
-        r = proc.poll()
-        log.info("stop_new_stream_notification(%s) exit code=%s", proc, r)
-        if r is not None:
-            #already ended
-            return
-        try:
-            proc.terminate()
-        except Exception:
-            log("failed to stop stream notification %s", proc)
-
-
     def parse_client_caps(self, c):
         self.wants_sound = c.boolget("wants_sound", True)
         self.pulseaudio_id = c.strget("sound.pulseaudio.id")
         self.pulseaudio_cookie_hash = c.strget("sound.pulseaudio.cookie-hash")
         self.pulseaudio_server = c.strget("sound.pulseaudio.server")
-        self.sound_decoders = c.strlistget("sound.decoders", [])
-        self.sound_encoders = c.strlistget("sound.encoders", [])
+        self.sound_decoders = c.strtupleget("sound.decoders", [])
+        self.sound_encoders = c.strtupleget("sound.encoders", [])
         self.sound_receive = c.boolget("sound.receive")
         self.sound_send = c.boolget("sound.send")
         self.sound_bundle_metadata = c.boolget("sound.bundle-metadata")
@@ -96,7 +77,7 @@ class AudioMixin(StubSourceMixin):
                  self.pulseaudio_id, self.pulseaudio_cookie_hash, self.pulseaudio_server,
                  self.sound_decoders, self.sound_encoders, self.sound_receive, self.sound_send)
 
-    def get_caps(self):
+    def get_caps(self) -> dict:
         if not self.wants_sound or not self.sound_properties:
             return {}
         sound_props = self.sound_properties.copy()
@@ -110,7 +91,7 @@ class AudioMixin(StubSourceMixin):
         return flatten_dict({"sound" : sound_props})
 
 
-    def audio_loop_check(self, mode="speaker"):
+    def audio_loop_check(self, mode="speaker") -> bool:
         log("audio_loop_check(%s)", mode)
         from xpra.sound.gstreamer_util import ALLOW_SOUND_LOOP, loop_warning_messages
         if ALLOW_SOUND_LOOP:
@@ -190,7 +171,7 @@ class AudioMixin(StubSourceMixin):
             return None
         try:
             from xpra.sound.wrapper import start_sending_sound
-            plugins = self.sound_properties.strlistget("plugins", [])
+            plugins = self.sound_properties.strtupleget("plugins")
             ss = start_sending_sound(plugins, self.sound_source_plugin,
                                      None, codec, volume, True, [codec],
                                      self.pulseaudio_server, self.pulseaudio_id)
@@ -219,7 +200,7 @@ class AudioMixin(StubSourceMixin):
     def sound_source_error(self, source, message):
         #this should be printed to stderr by the sound process already
         if source==self.sound_source:
-            log("sound source error: %s", message)
+            log("audio capture error: %s", message)
 
     def sound_source_exit(self, source, *args):
         log("sound_source_exit(%s, %s)", source, args)
@@ -266,15 +247,10 @@ class AudioMixin(StubSourceMixin):
                         "!", "audioconvert",
                         "!", sink]
                     import subprocess
-                    proc = subprocess.Popen(cmd, close_fds=True)
+                    proc = subprocess.Popen(cmd)
                     log("Popen(%s)=%s", cmd, proc)
                     from xpra.child_reaper import getChildReaper
                     getChildReaper().add_process(proc, "new-stream-sound", cmd, ignore=True, forget=True)
-                    def stop_new_stream_notification():
-                        if self.new_stream_timers.pop(proc, None):
-                            self.stop_new_stream_notification(proc)
-                    timer = self.timeout_add(NEW_STREAM_SOUND_STOP*1000, stop_new_stream_notification)
-                    self.new_stream_timers[proc] = timer
             except Exception:
                 pass
         log("new_stream(%s, %s)", sound_source, codec)
@@ -295,11 +271,8 @@ class AudioMixin(StubSourceMixin):
             update_av_sync()  #pylint: disable=not-callable
             #run it again after 10 seconds,
             #by that point the source info will actually be populated:
-            if PYTHON3:
-                from gi.repository import GLib as glib
-            else:
-                import glib
-            glib.timeout_add(10*1000, update_av_sync)
+            from gi.repository import GLib
+            GLib.timeout_add(10*1000, update_av_sync)
 
     def new_sound_buffer(self, sound_source, data, metadata, packet_metadata=None):
         log("new_sound_buffer(%s, %s, %s, %s) info=%s",
@@ -410,7 +383,7 @@ class AudioMixin(StubSourceMixin):
         return msg
 
     def sound_control_fadeout(self, delay_str=""):
-        assert self.sound_source, "no active sound source"
+        assert self.sound_source, "no active audio capture"
         delay = 1000
         if delay_str:
             delay = max(1, min(10*1000, int(delay_str)))
@@ -508,6 +481,8 @@ class AudioMixin(StubSourceMixin):
                     from xpra.sound.gstreamer_util import ENCODER_LATENCY, RECORD_PIPELINE_LATENCY
                     encoder_latency = RECORD_PIPELINE_LATENCY + ENCODER_LATENCY.get(ss.codec, 0)
                     cinfo = "%s " % ss.codec
+                #processing overhead
+                encoder_latency += 100
             except Exception as e:
                 encoder_latency = 0
                 log("failed to get encoder latency for %s: %s", ss.codec, e)
@@ -515,10 +490,10 @@ class AudioMixin(StubSourceMixin):
         return encoder_latency
 
 
-    def get_info(self):
+    def get_info(self) -> dict:
         return {"sound" : self.get_sound_info()}
 
-    def get_sound_info(self):
+    def get_sound_info(self) -> dict:
         def sound_info(supported, prop, codecs):
             i = {"codecs" : codecs}
             if not supported:
