@@ -79,6 +79,7 @@ class DesktopModel(WindowModelStub, WindowDamageHandler):
         "xid", "client-machine", "window-type",
         "shadow", "size-hints", "class-instance",
         "focused", "title", "depth", "icons",
+        "content-type",
         ]
     _dynamic_property_names = ["size-hints", "title", "icons"]
 
@@ -179,6 +180,8 @@ class DesktopModel(WindowModelStub, WindowDamageHandler):
             return True
         if prop=="class-instance":
             return ("xpra-desktop", "Xpra-Desktop")
+        if prop=="content-type":
+            return "desktop"
         return GObject.GObject.get_property(self, prop)
 
     def _screen_size_changed(self, screen):
@@ -299,10 +302,11 @@ class XpraDesktopServer(DesktopServerBaseClass):
 
     def server_init(self):
         X11ServerBase.server_init(self)
+        screenlog("server_init() randr=%s", self.randr)
         if self.randr:
             from xpra.x11.vfb_util import set_initial_resolution, DEFAULT_DESKTOP_VFB_RESOLUTION
             with xlog:
-                set_initial_resolution(DEFAULT_DESKTOP_VFB_RESOLUTION)
+                set_initial_resolution(self.initial_resolution or DEFAULT_DESKTOP_VFB_RESOLUTION)
 
     def x11_init(self):
         X11ServerBase.x11_init(self)
