@@ -9,11 +9,11 @@
 import os
 import signal
 import math
-from collections import deque, namedtuple
-from gi.repository import GObject, Gtk, Gdk
+from collections import deque
+from gi.repository import GObject, Gtk, Gdk, GdkX11
 
 from xpra.version_util import XPRA_VERSION
-from xpra.util import updict, rindex, envbool, envint, typedict, WORKSPACE_NAMES
+from xpra.util import updict, rindex, envbool, envint, typedict, AdHocStruct, WORKSPACE_NAMES
 from xpra.os_util import memoryview_to_bytes, strtobytes, bytestostr, monotonic_time
 from xpra.common import CLOBBER_UPGRADE, MAX_WINDOW_SIZE
 from xpra.server import server_features
@@ -24,9 +24,9 @@ from xpra.x11.common import Unmanageable
 from xpra.x11.gtk_x11.prop import prop_set
 from xpra.x11.gtk_x11.tray import get_tray_window, SystemTray
 from xpra.x11.gtk_x11.gdk_bindings import (
-   add_event_receiver,
-   get_children,
-   )
+    add_event_receiver,
+    get_children,
+    )
 from xpra.x11.bindings.window_bindings import X11WindowBindings #@UnresolvedImport
 from xpra.x11.bindings.keyboard_bindings import X11KeyboardBindings #@UnresolvedImport
 from xpra.x11.x11_server_base import X11ServerBase
@@ -44,6 +44,7 @@ metadatalog = Logger("x11", "metadata")
 framelog = Logger("x11", "frame")
 eventlog = Logger("x11", "events")
 mouselog = Logger("x11", "mouse")
+screenlog = Logger("x11", "screen")
 
 X11Window = X11WindowBindings()
 X11Keyboard = X11KeyboardBindings()
@@ -209,10 +210,12 @@ class XpraServer(GObject.GObject, X11ServerBase):
 
     def server_init(self):
         X11ServerBase.server_init(self)
-        if not self.clobber and self.randr:
+        screenlog("server_init() clobber=%s, randr=%s, initial_resolution=%s",
+                  self.clobber, self.randr, self.initial_resolution)
+        if self.randr and (self.initial_resolution or not self.clobber):
             from xpra.x11.vfb_util import set_initial_resolution, DEFAULT_VFB_RESOLUTION
             with xlog:
-                set_initial_resolution(DEFAULT_VFB_RESOLUTION)
+                set_initial_resolution(self.initial_resolution or DEFAULT_VFB_RESOLUTION)
 
     def server_ready(self):
         if not X11Window.displayHasXComposite():
@@ -244,10 +247,12 @@ class XpraServer(GObject.GObject, X11ServerBase):
                 with xsync:
                     self.root_overlay = X11Window.XCompositeGetOverlayWindow(xid)
                     if self.root_overlay:
-                        #ugly: API expects a window object with a ".xid"
-                        X11WindowModel = namedtuple("X11WindowModel", "xid")
-                        root_overlay = X11WindowModel(xid=self.root_overlay)
-                        prop_set(root_overlay, "WM_TITLE", "latin1", "RootOverlay")
+                        #ugly: API expects a window object with a ".get_xid()" method
+                        window = AdHocStruct()
+                        def get_xid():
+                            return self.root_overlay
+                        window.get_xid = root.get_xid
+                        prop_set(window, "WM_TITLE", "latin1", "RootOverlay")
                         X11Window.AllowInputPassthrough(self.root_overlay)
             except Exception as e:
                 log("XCompositeGetOverlayWindow(%#x)", xid, exc_info=True)
@@ -546,7 +551,7 @@ class XpraServer(GObject.GObject, X11ServerBase):
         window.managed_connect("motion", self._motion_signaled)
         window.managed_connect("x11-property-changed", self._x11_property_changed)
         if not window.is_tray():
-            window.managed_connect("raised", self._raised_window)
+            window.managed_connect("restack", self._restack_window)
             window.managed_connect("initiate-moveresize", self._initiate_moveresize)
         window_prop_set = getattr(window, "prop_set", None)
         if window_prop_set:
@@ -831,16 +836,17 @@ class XpraServer(GObject.GObject, X11ServerBase):
         wsources[0].initiate_moveresize(wid, window, *event.data)
 
 
-    def _raised_window(self, window, event):
+    def _restack_window(self, window, detail, sibling):
         wid = self._window_to_id[window]
-        windowlog("raised window: %s (%s) wid=%s, current focus=%s", window, event, wid, self._has_focus)
+        focuslog("restack window(%s) wid=%s, current focus=%s", (window, detail, sibling), wid, self._has_focus)
         if self.last_raised!=wid:
+            #ensure we will raise the window for the next pointer event
             self.last_raised = None
-        if self._has_focus==wid:
+        if detail==0 and self._has_focus==wid:  #Above=0
             return
         for ss in self._server_sources.values():
             if isinstance(ss, WindowsMixin):
-                ss.raise_window(wid, window)
+                ss.restack_window(wid, window, detail, sibling)
 
 
     def _set_window_state(self, proto, wid, window, new_window_state):
@@ -1048,26 +1054,19 @@ class XpraServer(GObject.GObject, X11ServerBase):
         if timer:
             return  #we already have one pending
         def damage():
-            try:
-                del self.configure_damage_timers[wid]
-            except KeyError:
-                pass
+            self.configure_damage_timers.pop(wid, None)
             window = self._lookup_window(wid)
             if window and window.is_managed():
                 self.refresh_window(window)
         self.configure_damage_timers[wid] = self.timeout_add(CONFIGURE_DAMAGE_RATE, damage)
 
     def cancel_configure_damage(self, wid):
-        timer = self.configure_damage_timers.get(wid)
+        timer = self.configure_damage_timers.pop(wid, None)
         if timer:
-            try:
-                del self.configure_damage_timers[wid]
-            except KeyError:
-                pass
             self.source_remove(timer)
 
     def cancel_all_configure_damage(self):
-        timers = self.configure_damage_timers.values()
+        timers = tuple(self.configure_damage_timers.values())
         self.configure_damage_timers = {}
         for timer in timers:
             self.source_remove(timer)
@@ -1160,8 +1159,8 @@ class XpraServer(GObject.GObject, X11ServerBase):
     #
 
     def update_root_overlay(self, window, x, y, image):
-        overlaywin = Gdk.window_foreign_new(self.root_overlay)
-        gc = overlaywin.new_gc()
+        display = Gdk.Display.get_default()
+        overlaywin = GdkX11.X11Window.foreign_new_for_display(display, self.root_overlay)
         wx, wy = window.get_property("geometry")[:2]
         #FIXME: we should paint the root overlay directly
         # either using XCopyArea or XShmPutImage,
@@ -1170,29 +1169,39 @@ class XpraServer(GObject.GObject, X11ServerBase):
         height = image.get_height()
         rowstride = image.get_rowstride()
         img_data = image.get_pixels()
-        if image.get_pixel_format().startswith("BGR"):
-            try:
-                from xpra.codecs.argb.argb import unpremultiply_argb, bgra_to_rgba  #@UnresolvedImport
-                if image.get_pixel_format()=="BGRA":
-                    img_data = unpremultiply_argb(img_data)
-                img_data = bgra_to_rgba(img_data)
-            except:
-                pass
-        img_data = memoryview_to_bytes(img_data)
-        has_alpha = image.get_pixel_format().find("A")>=0
-        log("update_root_overlay%s painting rectangle %s", (window, x, y, image), (wx+x, wy+y, width, height))
-        if has_alpha:
-            import cairo
-            pixbuf = get_pixbuf_from_data(img_data, True, width, height, rowstride)
-            cr = overlaywin.cairo_create()
-            cr.new_path()
-            cr.rectangle(wx+x, wy+y, width, height)
-            cr.clip()
-            cr.set_source_pixbuf(pixbuf, wx+x, wy+y)
-            cr.set_operator(cairo.OPERATOR_OVER)
-            cr.paint()
+        rgb_format = image.get_pixel_format()
+        from xpra.codecs.argb.argb import ( #@UnresolvedImport
+            unpremultiply_argb, bgra_to_rgba, bgra_to_rgbx, r210_to_rgbx, bgr565_to_rgbx  #@UnresolvedImport
+            )
+        from cairo import OPERATOR_OVER, OPERATOR_SOURCE  #pylint: disable=no-name-in-module
+        log("update_root_overlay%s rgb_format=%s, img_data=%i (%s)",
+                 (window, x, y, image), rgb_format, len(img_data), type(img_data))
+        operator = OPERATOR_SOURCE
+        if rgb_format=="BGRA":
+            img_data = unpremultiply_argb(img_data)
+            img_data = bgra_to_rgba(img_data)
+            operator = OPERATOR_OVER
+        elif rgb_format=="BGRX":
+            img_data = bgra_to_rgbx(img_data)
+        elif rgb_format=="r210":
+            #lossy...
+            img_data = r210_to_rgbx(img_data, width, height, rowstride, width*4)
+            rowstride = width*4
+        elif rgb_format=="BGR565":
+            img_data = bgr565_to_rgbx(img_data)
+            rowstride *= 2
         else:
-            overlaywin.draw_rgb_32_image(gc, wx+x, wy+y, width, height, Gdk.RGB_DITHER_NONE, img_data, rowstride)
+            raise Exception("xync-xvfb root overlay paint code does not handle %s pixel format" % image.get_pixel_format())
+        img_data = memoryview_to_bytes(img_data)
+        log("update_root_overlay%s painting rectangle %s", (window, x, y, image), (wx+x, wy+y, width, height))
+        pixbuf = get_pixbuf_from_data(img_data, True, width, height, rowstride)
+        cr = overlaywin.cairo_create()
+        cr.new_path()
+        cr.rectangle(wx+x, wy+y, width, height)
+        cr.clip()
+        Gdk.cairo_set_source_pixbuf(cr, pixbuf, wx+x, wy+y)
+        cr.set_operator(operator)
+        cr.paint()
         image.free()
 
     def repaint_root_overlay(self):
@@ -1213,7 +1222,8 @@ class XpraServer(GObject.GObject, X11ServerBase):
     def do_repaint_root_overlay(self):
         self.repaint_root_overlay_timer = None
         root_width, root_height = self.get_root_window_size()
-        overlaywin = Gdk.window_foreign_new(self.root_overlay)
+        display = Gdk.Display.get_default()
+        overlaywin = GdkX11.X11Window.foreign_new_for_display(display, self.root_overlay)
         log("overlaywin: %s", overlaywin.get_geometry())
         cr = overlaywin.cairo_create()
         def fill_grey_rect(shade, x, y, w, h):
@@ -1335,6 +1345,7 @@ class XpraServer(GObject.GObject, X11ServerBase):
                 with xsync:
                     img = window.get_image(0, 0, w, h)
             except XError:
+                log("%s.get_image%s", window, (0, 0, w, h), exc_info=True)
                 log.warn("screenshot: window %s could not be captured", wid)
                 continue
             if img is None:
