@@ -17,13 +17,14 @@ from xpra.x11.bindings.window_bindings import X11WindowBindings #@UnresolvedImpo
 from xpra.gtk_common.error import XError, xswallow, xsync, xlog, trap, verify_sync
 from xpra.gtk_common.gtk_util import get_default_root_window
 from xpra.server.server_uuid import save_uuid, get_uuid
+from xpra.x11.vfb_util import parse_resolution
 from xpra.x11.fakeXinerama import find_libfakeXinerama, save_fakeXinerama_config, cleanup_fakeXinerama
 from xpra.x11.gtk_x11.prop import prop_get, prop_set
 from xpra.x11.gtk_x11.gdk_display_source import close_gdk_display_source
 from xpra.x11.gtk_x11.gdk_bindings import init_x11_filter, cleanup_x11_filter, cleanup_all_event_receivers
 from xpra.common import MAX_WINDOW_SIZE
 from xpra.os_util import monotonic_time, strtobytes
-from xpra.util import typedict, iround, envbool, XPRA_DPI_NOTIFICATION_ID
+from xpra.util import typedict, iround, envbool, first_time, XPRA_DPI_NOTIFICATION_ID
 from xpra.net.compression import Compressed
 from xpra.server.gtk_server_base import GTKServerBase
 from xpra.x11.xkbhelper import clean_keyboard_state
@@ -88,7 +89,9 @@ class X11ServerCore(GTKServerBase):
         self.keys_pressed = {}
         self.last_mouse_user = None
         self.libfakeXinerama_so = None
+        self.initial_resolution = None
         self.x11_filter = False
+        self.randr_sizes_added = []
         GTKServerBase.__init__(self)
         log("XShape=%s", X11Window.displayHasXShape())
 
@@ -105,7 +108,11 @@ class X11ServerCore(GTKServerBase):
         super().server_init()
 
     def do_init(self, opts):
-        self.randr = opts.resize_display
+        try:
+            self.initial_resolution = parse_resolution(opts.resize_display)
+        except ValueError:
+            pass
+        self.randr = bool(self.initial_resolution) or not (opts.resize_display in FALSE_OPTIONS)
         self.randr_exact_size = False
         self.fake_xinerama = "no"      #only enabled in seamless server
         self.current_xinerama_config = None
@@ -316,7 +323,7 @@ class X11ServerCore(GTKServerBase):
                     "touchpad-device"              : bool(self.touchpad_device),
                     })
             if self.randr:
-                sizes = RandR.get_xrr_screen_sizes()
+                sizes = self.get_all_screen_sizes()
                 if len(sizes)>1:
                     capabilities["screen-sizes"] = sizes
             if self.default_cursor_image and source.wants_default_cursor:
@@ -365,7 +372,7 @@ class X11ServerCore(GTKServerBase):
         #randr:
         if self.randr:
             with xlog:
-                sizes = RandR.get_xrr_screen_sizes()
+                sizes = self.get_all_screen_sizes()
                 if len(sizes)>=0:
                     sinfo["randr"] = {
                         ""          : True,
@@ -478,10 +485,19 @@ class X11ServerCore(GTKServerBase):
         return (cursor_image, cursor_sizes)
 
 
+    def get_all_screen_sizes(self):
+        #workaround for #2910: the resolutions we add are not seen by XRRSizes!
+        # so we keep track of the ones we have added ourselves:
+        sizes = list(RandR.get_xrr_screen_sizes())
+        for w, h in self.randr_sizes_added:
+            if (w, h) not in sizes:
+                sizes.append((w, h))
+        return tuple(sizes)
+
     def get_max_screen_size(self):
         max_w, max_h = self.root_window.get_geometry()[2:4]
         if self.randr:
-            sizes = RandR.get_xrr_screen_sizes()
+            sizes = self.get_all_screen_sizes()
             if len(sizes)>=1:
                 for w,h in sizes:
                     max_w = max(max_w, w)
@@ -540,7 +556,7 @@ class X11ServerCore(GTKServerBase):
     def do_get_best_screen_size(self, desired_w, desired_h, bigger=True):
         if not self.randr:
             return desired_w, desired_h
-        screen_sizes = RandR.get_xrr_screen_sizes()
+        screen_sizes = self.get_all_screen_sizes()
         if (desired_w, desired_h) in screen_sizes:
             return desired_w, desired_h
         if self.randr_exact_size:
@@ -551,14 +567,16 @@ class X11ServerCore(GTKServerBase):
                         #we have to wait a little bit
                         #to make sure that everything sees the new resolution
                         #(ideally this method would be split in two and this would be a callback)
+                        self.randr_sizes_added.append(v)
                         import time
                         time.sleep(0.5)
                         return v
             except XError as e:
+                screenlog("add_screen_size(%s, %s)", desired_w, desired_h, exc_info=True)
                 screenlog.warn("Warning: failed to add resolution %ix%i:", desired_w, desired_h)
                 screenlog.warn(" %s", e)
             #re-query:
-            screen_sizes = RandR.get_xrr_screen_sizes()
+            screen_sizes = self.get_all_screen_sizes()
         #try to find the best screen size to resize to:
         new_size = None
         closest = {}
@@ -652,7 +670,7 @@ class X11ServerCore(GTKServerBase):
                 #the change! (ugly! but this works)
                 with xsync:
                     temp = {}
-                    for tw,th in RandR.get_xrr_screen_sizes():
+                    for tw,th in self.get_all_screen_sizes():
                         if tw!=w or th!=h:
                             #use the number of extra pixels as key:
                             #(so we can choose the closest resolution)
@@ -676,7 +694,7 @@ class X11ServerCore(GTKServerBase):
                 #we can use XRRSetScreenSize:
                 try:
                     with xsync:
-                        RandR.xrr_set_screen_size(w, h, self.xdpi or self.dpi, self.ydpi or self.dpi)
+                        RandR.xrr_set_screen_size(w, h, self.xdpi or self.dpi or 96, self.ydpi or self.dpi or 96)
                 except XError:
                     screenlog("XRRSetScreenSize failed", exc_info=True)
             screenlog("calling RandR.get_screen_size()")
@@ -724,7 +742,9 @@ class X11ServerCore(GTKServerBase):
     def notify_dpi_warning(self, body):
         sources = tuple(self._server_sources.values())
         if len(sources)==1:
-            sources[0].may_notify(XPRA_DPI_NOTIFICATION_ID, "DPI Issue", body, icon_name="font")
+            ss = sources[0]
+            if first_time("DPI-warning-%s" % ss.uuid):
+                sources[0].may_notify(XPRA_DPI_NOTIFICATION_ID, "DPI Issue", body, icon_name="font")
 
 
     def _process_server_settings(self, _proto, packet):
@@ -805,10 +825,7 @@ class X11ServerCore(GTKServerBase):
             return
         wid = 0
         if event.window!=get_default_root_window() and event.window_model is not None:
-            try:
-                wid = self._window_to_id[event.window_model]
-            except KeyError:
-                pass
+            wid = self._window_to_id.get(event.window_model, 0)
         log("_bell_signaled(%s,%r) wid=%s", wm, event, wid)
         from xpra.server.source.windows_mixin import WindowsMixin
         for ss in self._server_sources.values():
@@ -945,7 +962,7 @@ class X11ServerCore(GTKServerBase):
             if wid==self.get_focus():
                 ss.user_event()
 
-    def do_process_button_action(self, proto, wid, button, pressed, pointer, modifiers, _buttons=[], deviceid=-1, *_args):
+    def do_process_button_action(self, proto, wid, button, pressed, pointer, modifiers, _buttons=(), deviceid=-1, *_args):
         self._update_modifiers(proto, wid, modifiers)
         #TODO: pass extra args
         if self._process_mouse_common(proto, wid, pointer, deviceid):
